--
-- These examples have been extracted from the Fran user guide.
--

module Examples where

import Fran    -- Basic Fran functionality

example1 t = circ
-- Or use `disp example1'

circ :: ImageB

circ = bigger (sin time) (withColor red circle)

----------------------------------------------------------------

ball1, ball2, movingBall1, movingBall2 :: ImageB

ball1 = bigger 0.3 (withColor red circle)

movingBall1 = move (point2XY 0 wiggle) ball1

ball2 = bigger 0.4 (withColor blue circle)

movingBall2 = move (point2XY wiggle 0) ball2


example2 t = movingBall1 `over` movingBall2

----------------------------------------------------------------

example3 t = move (mouse t) ball1

----------------------------------------------------------------

color4 :: Time -> ColorB
color4 t = red `untilB` lbp t -=> green

example4 t = withColor (color4 t) circle

----------------------------------------------------------------

color5 :: Time -> ColorB
color5 t = red `untilB` lbp t *=>
             (\t' -> green `untilB` lbp t' *=> color5)

example5 t = withColor (color5 t) circle

----------------------------------------------------------------

toggle :: Time -> Behavior a -> Behavior a -> Behavior a
toggle t v1 v2 = v1 `untilB` lbp t *=> (\t' -> toggle t' v2 v1)

ball61 t = withColor (toggle t red green) (bigger 0.4 circle) 
ball62 t = withColor (toggle t green red) (bigger 0.6 circle) 

example6 t = ball61 t `over` ball62 t

----------------------------------------------------------------

color7 :: Time -> ColorB
color7 t = red `untilB` ((lbp t *=>
                            (\t' -> green `untilB` lbp t' *=> color7))
                         .|. 
                         (rbp t *=>
                             (\t' -> blue `untilB` rbp t' *=> color7)))

example7 t = withColor (color7 t) circle

----------------------------------------------------------------

gravity8 :: RealB

gravity8 = -0.1

velocity8, position8 :: Time -> RealB

velocity8 t0 = integral gravity8 t0

position8 t0 = integral (velocity8 t0) t0

example8 t = withColor red (moveXY 0 (position8 t) (bigger 0.1 circle))

----------------------------------------------------------------

p9     :: Time -> Point2B -> Point2B
p9 t p = p `untilB` lbp t `snapshot` (mouse t) +=>
                             \t' (_,p') -> p9 t' (constantB p')

-- A more terse style:
-- p9 t  p = p `untilB` lbp t `snapshot` (mouse t) ==> snd +=> p9

example9 t = withColor red (move (p9 t origin2) (bigger 0.1 circle)) 

----------------------------------------------------------------

-- A circling ball, r = radius, c = color
cball r c = withColor c (moveXY (r*sin time) (r*cos time) 
                               (bigger 0.1 circle))
-- using polar form would be perhaps clearer:
-- cball r c = withColor c (move (vector2Polar r time) (bigger 0.1 circle))

example10 t = cball 0.5 red `over` 
              (cball 0.7 blue `timeTransform` (2*time)) `over`
              (cball 0.9 green `timeTransform` (time + wiggle))

----------------------------------------------------------------

gravity11 :: RealB
gravity11 = -0.6

velocity11 :: Time -> RealB -> RealB
velocity11 t0 v0 = v0 + integral gravity11 t0

-- Assume floor is at y = -1
position11 :: Time -> RealB -> RealB -> RealB
position11 t0 x0 v0 = p where
   v = velocity11 t0 v0 
   p = x0 + integral v t0 `untilB`
              predicate (p <* (-1)) t0 `snapshot` v +=>
                  \t' (_,v') -> position11 t' (-1) (constantB (-v' * 0.9))
-- Bounces with only 0.9 of the velocity each time

example11 t = withColor red (moveXY 0 (position11 t 0 (-0.1))
                                   (bigger 0.1 circle))

----------------------------------------------------------------

im12 = unitBBoxed2 (moveXY 0.5 0.5  (bigger 0.3 circle)
                    `over` moveXY (wiggle/4) 0 (bigger 0.5 im12))

example12 t = withColor red im12

-- These make the examples easier to view 

run i 
  | 1 <= i && i <= length examples
  = disp (fst (examples !! (i-1)))
  | otherwise
  = return ()

examples = [(example1,"Example 1"),
    	    (example2,"Example 2"),
	    (example3,"Example 3"),
	    (example4,"Example 4"),
	    (example5,"Example 5"),
            (example6,"Example 6"),
	    (example7,"Example 7"),
	    (example8,"Example 8"),
	    (example9,"Example 9"),
	    (example10,"Example 10"),
	    (example11,"Example 11"),
	    (example12,"Example 12")]

dispFps imF = disp (\t0 -> fpsImage t0 `over` imF t0)

-- Sequential demo sequencing
seqImF :: [(Time -> ImageB)] -> [(Time -> ImageB)] -> (Time -> ImageB)

seqImF (b:bs) r t0 =
  b t0 `untilB` (keyPress `suchThat` 
                   (\(ch,_) -> ch `elem` " N'" && not (null bs))) t0 *=>
                           seqImF bs (b:r)
            .|. (keyPress `suchThat` 
                   (\(ch,_) -> ch `elem` "P%" && not (null r))) t0 *=> 
                           seqImF (head r:b:bs) (tail r)
 where
  keyIn chars =
     keyPress `suchThat` (\(ch,_) -> ch `elem` chars)
 

allDemos  =  seqImF (map addLabel examples) []

addLabel (image, name) = \t -> (moveXY (-0.5) 0.9 $
                                withColor black $
                                stringIm name) `over` (image t)

-- An image of the fps behavior
fpsImage :: Time -> ImageB
fpsImage = withColor yellow . showIm . fps

main () = disp allDemos


