--
-- Fran   Lifted 2D point operations
--

module Point2B (
	Point2B, Point2,
	origin2,
	point2XY,    point2XYCoords,
	point2Polar, point2PolarCoords,
	distance2,   distance2Squared,
	(.+^), (.-^), (.-.),
	linearInterpolate2
	) where

import qualified Point2 as P
import Point2(Point2)
import Fuzzy
import Behavior
import BaseTypes
import Vector2B(Vector2B)

-- Hugs bug (?) workaround.  See comment in VectorSpaceB.
-- Set binding strength to that of + and -.
-- infix 4 .+^ , .-^ ,  .-.

type Point2B = Behavior P.Point2

origin2             :: Point2B                                   
point2XY            :: Behavior RealVal -> Behavior RealVal -> Point2B
point2Polar         :: Behavior Length  -> Behavior Radians -> Point2B
point2XYCoords      :: Point2B  -> Behavior (RealVal, RealVal)            
point2PolarCoords   :: Point2B  -> Behavior (Radians, Length )            
distance2           :: Point2B  -> Point2B  -> Behavior Length             
distance2Squared    :: Point2B  -> Point2B  -> Behavior Length             
linearInterpolate2  :: Point2B  -> Point2B  -> Behavior RealVal -> Point2B  
(.+^)               :: Point2B  -> Vector2B -> Point2B             
(.-^)               :: Point2B  -> Vector2B -> Point2B             
(.-.)               :: Point2B	-> Point2B  -> Vector2B            

origin2           = lift0 P.origin2
point2XY          = lift2 P.point2XY (noI "point2XY")
point2Polar       = lift2 P.point2Polar (noI "point2Polar")
point2XYCoords    = lift1 P.point2XYCoords (noI "point2XYCoords")
point2PolarCoords = lift1 P.point2PolarCoords (noI "point2PolarCoords")
distance2         = lift2 P.distance2 (noI "distance2")
distance2Squared  = lift2 P.distance2Squared (noI "distance2Squared")
linearInterpolate2 = lift3 P.linearInterpolate2 (noI "linearInterpolate2")
(.+^)             = lift2 (P..+^) (noI "(.+^)")
(.-^)             = lift2 (P..-^) (noI "(.-^)")
(.-.)             = lift2 (P..-.) (noI "(.-.)")
