--
-- Fran imaging utilities
--
-- Ad hoc collection of definitions. These are here for introductory use,
-- to make it easier to do some simple things. I'd like to make it so that
-- kids can use this simple vocabulary.
--
-- Last modified Sun Nov 10 16:18:49 1996

module UtilsB (move, moveXY, bigger, biggerXY, smaller, smallerXY,
               turnLeft, turnRight, wiggle, waggle, wiggleRange, 
               later, earlier, faster, slower, 
               stringIm, stringBIm, 
               showIm, atRate,
               viewStretch ) where

import qualified StaticTypes as S
import Force
import BaseTypes
import Behavior
import VectorSpaceB
import Event
import Vector2B
import Point2B
import ColorB
import TextB
import Transform2B
import ImageB
import Integral
import Until
import Interaction


-- move an object to a point
move       ::  (Translateable2 a,Transformable2 b) =>
                   Behavior a -> Behavior b -> Behavior b
moveXY     :: Transformable2 a =>
                RealB -> RealB -> Behavior a -> Behavior a 

-- Multiply size
bigger     :: Transformable2 a => RealB -> Behavior a -> Behavior a
biggerXY   :: Transformable2 a => RealB -> RealB -> Behavior a -> Behavior a 

-- divide size
smaller    :: Transformable2 a => RealB -> Behavior a -> Behavior a
smallerXY  :: Transformable2 a => RealB -> RealB -> Behavior a -> Behavior a 

-- rotate image; fraction (1.0 = 360 degrees) left / right about the origin
turnLeft   :: Transformable2 a => RealB -> Behavior a -> Behavior a 
turnRight  :: Transformable2 a => RealB -> Behavior a -> Behavior a 

-- Oscillates between -1 and 1
wiggle     :: RealB
-- Ditto, but delayed 0.5 sec
waggle     :: RealB
-- Scaled wiggle, same period
wiggleRange :: RealB -> RealB -> RealB

-- Time transforming of behaviors
later       :: TimeB -> Behavior a -> Behavior a
earlier     :: TimeB -> Behavior a -> Behavior a
faster      :: RealB -> Behavior a -> Behavior a
slower      :: RealB -> Behavior a -> Behavior a

-- String to image conversions
stringIm    :: String -> ImageB
stringBIm   :: Behavior String -> ImageB

-- Continuous show, rendered into an image (good for debugging)
showIm      :: Show a => Behavior a -> ImageB

-- Synonym for integral
atRate      :: (Forceable v, VectorSpace v) => Behavior v -> Time -> Behavior v

viewStretch :: Vector2B -> Time -> ImageB -> ImageB

-- Move

move dp thing = translate2 dp *% thing
moveXY dx dy thing   = translate2 (vector2XY dx dy) *% thing

-- Resize
bigger sc thing = uscale2 sc *% thing

smaller sc = bigger (1/sc)

biggerXY scx scy im = scale2 (vector2XY scx scy) *% im

smallerXY scx scy = biggerXY (1/scx) (1/scy)

turnLeft frac im = rotate2 (frac * pi) *% im

turnRight frac = turnLeft (-frac)

-- Oscillates between -1 and 1
wiggle  = sin (pi * time)

-- Ditto, but off phase
waggle = later 0.5 wiggle

wiggleRange lo hi =
 lo + (hi-lo) * (wiggle+1)/2

-- Shift a behavior to be later or earlier, faster or slower
-- ADR: I changed this to be a TimeB (used to be Time)
--      required changes in samples.hs
later dt b = b `timeTransform` (time - dt)
earlier dt = later (-dt)

faster x b = b `timeTransform` (time * x)
slower x   = faster (1/x)

-- Continuous show, rendered into an image

showIm = stringBIm . showB
stringIm = stringBIm . lift0
stringBIm = renderedText . simpleText


-- Synonym

atRate = integral


-- More specialized 

-- Given an image and a canonical size, stretch the image so that the size
-- maps exactly onto the window view size.

viewStretch size t0 =
  biggerXY (wWidth  / iWidth ) (wHeight / iHeight)
  where
    (wWidth, wHeight) = pairBSplit (vector2XYCoords (viewSize t0))
    (iWidth, iHeight) = pairBSplit (vector2XYCoords size)

