-----------------------------------------------------------------------------
-- This implements "buffered channels" exactly as described in the paper
--
--   "Concurrent Haskell"
--   Simon Peyton Jones, Andrew Gordon and Sigbjorn Finne.
--   In Proceedings of the ACM Symposium on Principles of Programming
--   Languages,St Petersburg Beach, Florida, January 1996. 
--   http://www.dcs.gla.ac.uk/fp/authors/Simon_Peyton_Jones/
--     concurrent-haskell.ps
--
-----------------------------------------------------------------------------

module Channel(
       Channel(..),
       newChan, putChan, getChan, dupChan
       ) where

import MVar(MVar, newMVar, takeMVar, putMVar)

newChan :: IO (Channel a)
putChan :: Channel a -> a -> IO ()
getChan :: Channel a -> IO a

-- To support multicast operation:
-- Make copies of channel which will see any values subsequently written
-- to original channel.
-- To use, each listener makes their own copy of the channel.
dupChan :: Channel a -> IO (Channel a)

---------------- Implementation ----------------

type Channel a = (MVar (Stream a),   -- Read
                  MVar (Stream a))   -- Write

type Stream a = MVar (Item a)
data Item a = Item a (Stream a)

newChan = do
    read  <- newMVar
    write <- newMVar
    hole  <- newMVar
    putMVar read hole     
    putMVar write hole    
    return (read, write)

putChan (read, write) val = do
    new_hole <- newMVar 
    old_hole <- takeMVar write
    putMVar write new_hole   
    putMVar old_hole (Item val new_hole)

getChan (read, write) = do
    cts <- takeMVar read
    res@(Item val new) <- takeMVar cts
    putMVar cts res  
    putMVar read new 
    return val

dupChan (read, write) = do
    new_read <- newMVar       
    hole     <- takeMVar write
    putMVar write hole    
    putMVar new_read hole 
    return (new_read, write)

-- ToDo: ungetChan :: Channel a -> a -> IO a


