################################################################
# Miscellaneous drawing functions
################################################################

%errfun False moveToEx   :: HDC -> INT -> INT -> IO POINT using $err = MoveToEx($0,$1,$2,&$r0)
%errfun False LineTo     :: HDC -> INT -> INT -> IO ()
%errfun False polyline   :: HDC -> ListPOINT  -> IO ()    using $err = Polyline($0,$1,$1_size)
%errfun False polylineTo :: HDC -> ListPOINT  -> IO ()    using $err = PolylineTo($0,$1,$1_size)
%errfun False polygon    :: HDC -> ListPOINT  -> IO ()    using $err = Polygon($0,$1,$1_size)

# Raises an error if not given exactly 3n+1 points
# Maybe we should give it a more structured type to guarantee this?
%errfun False polyBezier   :: HDC -> ListPOINT -> IO ()   using $err = PolyBezier($0,$1,$1_size)
%errfun False polyBezierTo :: HDC -> ListPOINT -> IO ()   using $err = PolyBezierTo($0,$1,$1_size)

# ToDo: PolyPolygon, PolyPolyLine, PolyDraw, PolyTextOut

%errfun False fillRect   :: HDC -> RECT -> HBRUSH -> IO () using $err = FillRect($0,&$1,$2)
%errfun False frameRect  :: HDC -> RECT -> HBRUSH -> IO () using $err = FrameRect($0,&$1,$2)
%errfun False invertRect :: HDC -> RECT -> IO ()           using $err = InvertRect($0,&$1)
%errfun False Rectangle  :: HDC -> INT -> INT -> INT -> INT -> IO ()
%errfun False RoundRect  :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
%errfun False Ellipse    :: HDC -> INT -> INT -> INT -> INT -> IO ()
%errfun False Arc        :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
%errfun False ArcTo      :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
%errfun False AngleArc   :: HDC -> INT -> INT -> DWORD -> Float -> Float -> IO ()
%errfun False Chord      :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()
%errfun False Pie        :: HDC -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> INT -> IO ()


################################################################
# Device Contexts
################################################################

# GetObjectType omitted (I don't believe we need it)


################ Region Types ################
# Returned by many region operations

%newtype : RegionType : INT : (Eq)
%error   : BadRegion  : RegionType $0 : (ERROR == $0) : %constructor(WindowsError,$0);

%const RegionType ERROR               
%const RegionType NULLREGION          
%const RegionType SIMPLEREGION        
%const RegionType COMPLEXREGION       

################ Region Combination Mode ################

%newtype : ClippingMode : INT : (Eq)

%const ClippingMode RGN_AND             
%const ClippingMode RGN_OR              
%const ClippingMode RGN_XOR             
%const ClippingMode RGN_DIFF            
%const ClippingMode RGN_COPY            

################ GetStockObject ################

%newtype : StockBrush   : INT : ()
%newtype : StockPen     : INT : ()
%newtype : StockFont    : INT : ()
%newtype : StockPalette : INT : ()

%const StockBrush   WHITE_BRUSH         
%const StockBrush   LTGRAY_BRUSH        
%const StockBrush   GRAY_BRUSH          
%const StockBrush   DKGRAY_BRUSH        
%const StockBrush   BLACK_BRUSH         
%const StockBrush   NULL_BRUSH          
%const StockBrush   HOLLOW_BRUSH        # NULL_BRUSH
		    
%const StockPen     WHITE_PEN           
%const StockPen     BLACK_PEN           
%const StockPen     NULL_PEN            
		    	     
%const StockFont    OEM_FIXED_FONT      
%const StockFont    ANSI_FIXED_FONT     
%const StockFont    ANSI_VAR_FONT       
%const StockFont    SYSTEM_FONT         
%const StockFont    DEVICE_DEFAULT_FONT 
%const StockFont    SYSTEM_FIXED_FONT   

%const StockPalette DEFAULT_PALETTE     

# ToDo: I don't see how these can fail - so we can eliminate the Maybe
# and I'll bet I could drop the IO too.

# ToDo: I could save code size here by generating a single %asm
# and writing 4 different primitive declarations.

%errfun NullHANDLE getStockBrush   :: StockBrush   -> IO HBRUSH   using $r0 = GetStockObject($0);

# The manual says:
#   "You should use stock fonts only if the mapping mode for your
#    application's device context is MM_TEXT."
%errfun NullHANDLE getStockFont    :: StockFont    -> IO HFONT    using $r0 = GetStockObject($0);

# ToDo: since there's only one stock palette, I could drop the argument?
%errfun NullHANDLE getStockPalette :: StockPalette -> IO HPALETTE using $r0 = GetStockObject($0);
%errfun NullHANDLE getStockPen     :: StockPen     -> IO HPEN     using $r0 = GetStockObject($0);


################ DeleteObject ################

# Note: deleting a pattern brush doesn't delete enclosed bitmap
%errfun False deleteBrush   :: HBRUSH   -> IO () using $err = DeleteObject($0)
%errfun False deleteBitmap  :: HBITMAP  -> IO () using $err = DeleteObject($0)
%errfun False deleteFont    :: HFONT    -> IO () using $err = DeleteObject($0)
%errfun False deletePalette :: HPALETTE -> IO () using $err = DeleteObject($0)
%errfun False deletePen     :: HPEN     -> IO () using $err = DeleteObject($0)
%errfun False deleteRegion  :: HRGN     -> IO () using $err = DeleteObject($0)


################ GetObject ################

# not supported yet since we don't support the relevant structures yet



################ GetCurrentObject ################

%errfun NullHANDLE getCurrentBitmap  :: HDC -> IO HBITMAP  using $r0 = GetCurrentObject($0, OBJ_BITMAP)
%errfun NullHANDLE getCurrentBrush   :: HDC -> IO HBRUSH   using $r0 = GetCurrentObject($0, OBJ_BRUSH)
%errfun NullHANDLE getCurrentFont    :: HDC -> IO HFONT    using $r0 = GetCurrentObject($0, OBJ_FONT)
%errfun NullHANDLE getCurrentPalette :: HDC -> IO HPALETTE using $r0 = GetCurrentObject($0, OBJ_PAL)
%errfun NullHANDLE getCurrentPen     :: HDC -> IO HPEN     using $r0 = GetCurrentObject($0, OBJ_PEN)


################ SelectObject ################

# If you do this for a "memory device", any further operations on the
# "device" will modify the bitmap.  For this reason, it is an error
# to simultaneously select the same bitmap into more than one device.
%errfun NullHANDLE selectBitmap     :: HDC -> HBITMAP -> IO HBITMAP    using $r0 = SelectObject($0,$1);
%errfun NullHANDLE selectBrush      :: HDC -> HBRUSH  -> IO HBRUSH     using $r0 = SelectObject($0,$1);
%errfun NullHANDLE selectFont       :: HDC -> HFONT   -> IO HFONT      using $r0 = SelectObject($0,$1);
%errfun NullHANDLE selectPen        :: HDC -> HPEN    -> IO HPEN       using $r0 = SelectObject($0,$1);
%errfun BadRegion  selectRgn        :: HDC -> HRGN    -> IO RegionType using $r0 = (RegionType) SelectObject($0,$1);
%errfun BadRegion  SelectClipRgn    :: HDC -> MbHRGN  -> IO RegionType
%errfun BadRegion  SelectClipPath   :: HDC -> ClippingMode -> IO RegionType
%errfun BadRegion  ExtSelectClipRgn :: HDC -> HRGN    -> ClippingMode -> IO RegionType
%errfun NullHANDLE SelectPalette    :: HDC -> HPALETTE -> Bool -> IO HPALETTE

# The semantics of this operation are (deliberately?) vague
# Mostly used in connection with threads
%errfun False CancelDC :: HDC -> IO ()

%errfun NullHANDLE CreateCompatibleDC :: MbHDC -> IO HDC
%errfun False      DeleteDC           :: HDC   -> IO ()

# Undocumented Windows feature: GetDC(NULL) gives DC for entire screen
%errfun NullHANDLE GetDC       :: MbHWND -> IO HDC
%errfun NullHANDLE GetWindowDC :: MbHWND -> IO HWND
%errfun False      ReleaseDC   :: MbHWND -> HDC -> IO ()


%newtype : GetDCExFlags : DWORD : ()

%const GetDCExFlags DCX_WINDOW          
%const GetDCExFlags DCX_CACHE           
%const GetDCExFlags DCX_CLIPCHILDREN    
%const GetDCExFlags DCX_CLIPSIBLINGS    
%const GetDCExFlags DCX_PARENTCLIP      
%const GetDCExFlags DCX_EXCLUDERGN      
%const GetDCExFlags DCX_INTERSECTRGN    
%const GetDCExFlags DCX_LOCKWINDOWUPDATE
#%const GetDCExFlags DCX_USESTYLE        

# ToDo: should second arg have type MbRGN?
%errfun NullHANDLE GetDCEx    :: HWND -> HRGN -> GetDCExFlags -> IO HDC
%errfun False      GetDCOrgEx :: HDC  -> IO POINT using $err = GetDCOrgEx($0,&$r0)

################################################################

%newtype : GraphicsMode   : INT : (Eq)
%maybe   : MbGraphicsMode : GraphicsMode : $0 : 0

%const GraphicsMode GM_COMPATIBLE
%const GraphicsMode GM_ADVANCED

# ToDo: it isn't clear if this modifies the "LastError" flag or not
# so I've left this as a Maybe type
%fun GetGraphicsMode :: HDC -> IO MbGraphicsMode
%fun SetGraphicsMode :: HDC -> GraphicsMode -> IO MbGraphicsMode

# ToDo: use MbINT or something to capture error situations?
%errfun Zero  SaveDC    :: HDC -> IO INT
%errfun False RestoreDC :: HDC -> INT -> IO ()


# ToDo: define DevMode struct
#
# The 3rd parameter should always be NULL for Win32 applications
# %prim createDC :: String -> String -> LPDEVMODE -> IO MbHDC
# MbHDC r = CreateDC($0,$1,NULL,$2);
# %return(r);
#
# The 3rd parameter should always be NULL for Win32 applications
# %prim createIC :: String -> String -> LPDEVMODE -> IO MbHDC
# MbHDC r = CreateIC($0,$1,NULL,$2);
# %return(r);
#
# %fun ResetDC :: HDC -> LPDEVMODE -> IO MbHDC

# ToDo: DeviceCapabilities
# ToDo: DrawEscape
# ToDo: Escape
# ToDo: EnumObjects
# ToDo: EnumObjectsProc

################################################################
# Regions
################################################################

################ Arc Direction (not in Win32s) ################
# Affects: Arc, Chord, Pie, RoundRect, ArcTo, Ellipse, Rectangle

%newtype : ArcDirection   : INT : (Eq)
%maybe   : MbArcDirection : ArcDirection : $0 : 0
 
%const ArcDirection	AD_COUNTERCLOCKWISE
%const ArcDirection	AD_CLOCKWISE

%errfun Zero SetArcDirection :: HDC -> ArcDirection -> IO ArcDirection

################ PolyFill Mode ################

%newtype : PolyFillMode   : INT : (Eq)
%maybe   : MbPolyFillMode : PolyFillMode : $0 : 0

# PolyFill Modes
%const PolyFillMode ALTERNATE   
%const PolyFillMode WINDING     

%errfun Zero GetPolyFillMode :: HDC -> IO PolyFillMode
%errfun Zero SetPolyFillMode :: HDC -> PolyFillMode -> IO PolyFillMode

################ Region Functions ################

%errfun BadRegion CombineRgn :: HRGN -> HRGN -> HRGN -> ClippingMode -> IO RegionType
%errfun BadRegion OffsetRgn  :: HRGN -> INT -> INT -> IO RegionType
%errfun BadRegion GetRgnBox  :: HRGN -> LPRECT -> IO RegionType

%errfun NullHANDLE CreateEllipticRgn         :: INT -> INT -> INT -> INT -> IO HRGN
%errfun NullHANDLE CreateEllipticRgnIndirect :: LPRECT ->                   IO HRGN
%errfun NullHANDLE CreateRectRgn             :: INT -> INT -> INT -> INT -> IO HRGN
%errfun NullHANDLE CreateRectRgnIndirect     :: LPRECT ->                   IO HRGN
%errfun NullHANDLE CreateRoundRectRgn        :: INT -> INT -> INT -> INT -> INT -> INT -> IO HRGN
%errfun NullHANDLE CreatePolygonRgn          :: ListPOINT -> PolyFillMode -> IO HRGN using $r0 = CreatePolygonRgn($0,$0_size,$1)

# ToDo: ExtCreateRegion, GetRegionData
#
# %struct : RGNDATAHEADER : RGNDATAHEADER $0
#   DWORD : dwSize
#   DWORD : iType    # must be RDH_RECTANGLES
#   DWORD : nCount
#   DWORD : nRgnSize
#   RECT  : rcBound

%errfun BadRegion EqualRgn     :: HRGN -> HRGN -> IO Bool
%errfun False     FillRgn      :: HDC -> HRGN -> HBRUSH -> IO ()
%errfun False     InvertRgn    :: HDC -> HRGN -> IO ()
%errfun False     PaintRgn     :: HDC -> HRGN -> IO ()
%errfun False     FrameRgn     :: HDC -> HRGN -> HBRUSH -> INT -> INT -> IO ()
%fun              PtInRegion   :: HRGN -> INT -> INT -> IO Bool
%fun              RectInRegion :: HRGN -> RECT -> IO Bool using $r0 = RectInRegion($0,&$1)


################################################################
# Paths
################################################################

%errfun False      AbortPath         :: HDC -> IO ()
%errfun False      BeginPath         :: HDC -> IO ()
%errfun False      CloseFigure       :: HDC -> IO ()
%errfun False      EndPath           :: HDC -> IO ()
%errfun False      FillPath          :: HDC -> IO ()
%errfun False      FlattenPath       :: HDC -> IO ()
%errfun NullHANDLE PathToRegion      :: HDC -> IO HRGN
%errfun False      StrokeAndFillPath :: HDC -> IO ()
%errfun False      StrokePath        :: HDC -> IO ()
%errfun False      WidenPath         :: HDC -> IO ()
# SelectClipPath is defined with device contexts

# ToDo: GetPath

%errfun False GetMiterLimit :: HDC ->          IO FLOAT using $err = GetMiterLimit($0,&$r0)
%errfun False SetMiterLimit :: HDC -> FLOAT -> IO FLOAT using $err = SetMiterLimit($0,$1,&$r0)


################################################################
# Transformations
################################################################

%struct : XFORM : XFORM $0
  FLOAT : eM11
  FLOAT : eM12
  FLOAT : eM21
  FLOAT : eM12
  FLOAT : eDx
  FLOAT : eDy

################################################################
# Colours
################################################################

%newtype : COLORREF : Word32 : () : noTypedef

%fun rgb        :: BYTE -> BYTE -> BYTE -> COLORREF using $r0 = RGB($0,$1,$2)
%fun GetRValue  :: COLORREF -> BYTE
%fun GetGValue  :: COLORREF -> BYTE
%fun GetBValue  :: COLORREF -> BYTE
%fun PALETTERGB :: BYTE -> BYTE -> BYTE -> COLORREF

# ToDo: should this be abstract?
%newtype : PaletteIndex : WORD : ()

%fun PALETTEINDEX :: PaletteIndex -> COLORREF



################################################################
# Brushes
################################################################

# Note: brushes are not deleted when the application ends so we'd
# better be careful about deallocating them.

%newtype : HatchStyle : INT : (Eq)

# Hatch Styles
%const HatchStyle HS_HORIZONTAL 
%const HatchStyle HS_VERTICAL   
%const HatchStyle HS_FDIAGONAL  
%const HatchStyle HS_BDIAGONAL  
%const HatchStyle HS_CROSS      
%const HatchStyle HS_DIAGCROSS  

# x and y should be in range 0..7
%prim setBrushOrgEx    :: HDC -> INT -> INT -> IO (Maybe POINT)
POINT pt;
if (SetBrushOrgEx($0,$1,$2,&pt)) {
  %return(pt);
} else {
  %nothing();
}

%errfun False      getBrushOrgEx      :: HDC        -> IO POINT using $err = GetBrushOrgEx($0,&$r0);
%errfun NullHANDLE CreateSolidBrush   :: COLORREF   -> IO HBRUSH
%errfun NullHANDLE CreateHatchBrush   :: HatchStyle -> COLORREF -> IO HBRUSH
%errfun NullHANDLE CreatePatternBrush :: HBITMAP    -> IO HBRUSH

# ToDo: CreateBrushIndirect
# ToDo: CreateDIBPatternBrush, CreateDIBPatternBrushPt

################################################################
# Pens
################################################################

%newtype : PenStyle : INT : (Eq)
{{%
instance BitSet PenStyle where
  toWord32   = \ (PenStyle x) -> x
  fromWord32 = PenStyle
%}}

# Pick one of these
%const PenStyle	PS_SOLID            # default
%const PenStyle	PS_DASH             # -------  
%const PenStyle	PS_DOT              # .......  
%const PenStyle	PS_DASHDOT          # _._._._  
%const PenStyle	PS_DASHDOTDOT       # _.._.._  
%const PenStyle	PS_NULL        
%const PenStyle	PS_INSIDEFRAME 
%const PenStyle	PS_USERSTYLE   
%const PenStyle	PS_ALTERNATE   
%const PenStyle	PS_STYLE_MASK       # all the above

# "or" with one of these
%const PenStyle	PS_ENDCAP_ROUND     # default
%const PenStyle	PS_ENDCAP_SQUARE    
%const PenStyle	PS_ENDCAP_FLAT      
%const PenStyle	PS_ENDCAP_MASK      # all the above

# "or" with one of these
%const PenStyle	PS_JOIN_ROUND       # default
%const PenStyle	PS_JOIN_BEVEL       
%const PenStyle	PS_JOIN_MITER       
%const PenStyle	PS_JOIN_MASK        # all the above

# "or" with one of these
%const PenStyle	PS_COSMETIC         # default
%const PenStyle	PS_GEOMETRIC        
%const PenStyle	PS_TYPE_MASK        # all the above

%errfun NullHANDLE CreatePen    :: PenStyle -> INT -> COLORREF -> IO HPEN

# Not very well supported on Win'95
# %errfun NullHANDLE ExtCreatePen :: PenStyle -> INT -> LOGBRUSH -> [StyleBit] -> IO HPEN using ...

# ToDo: CreatePenIndirect

################################################################
# End
################################################################
