################################################################
# Registry
################################################################

%newtype : HKEY   : NearPtr : (Eq) : notypedef
%maybe   : MbHKEY : HKEY    : $0   : (HKEY) 0

%const HKEY HKEY_CLASSES_ROOT      
%const HKEY HKEY_CURRENT_USER      
%const HKEY HKEY_LOCAL_MACHINE     
%const HKEY HKEY_USERS             
%const HKEY HKEY_PERFORMANCE_DATA  
%const HKEY HKEY_CURRENT_CONFIG    
%const HKEY HKEY_DYN_DATA          

################################################################

%newtype : KeyRestoreFlags : DWORD : (Eq)

%const KeyRestoreFlags REG_WHOLE_HIVE_VOLATILE # Restore whole hive volatile
%const KeyRestoreFlags REG_REFRESH_HIVE        # Unwind changes to last flush
%const KeyRestoreFlags REG_NO_LAZY_FLUSH       # Never lazy flush this hive

################################################################

%newtype : RegType : DWORD : (Eq)

%const RegType REG_NONE                     # No value type
%const RegType REG_SZ                       # Unicode nul terminated string
%const RegType REG_EXPAND_SZ                # Unicode nul terminated string
              				    # (with environment variable references)
%const RegType REG_BINARY                   # Free form binary
%const RegType REG_DWORD                    # 32-bit number
%const RegType REG_DWORD_LITTLE_ENDIAN      # 32-bit number (same as REG_DWORD)
%const RegType REG_DWORD_BIG_ENDIAN         # 32-bit number
%const RegType REG_LINK                     # Symbolic Link (unicode)
%const RegType REG_MULTI_SZ                 # Multiple Unicode strings
%const RegType REG_RESOURCE_LIST            # Resource list in the resource map
%const RegType REG_FULL_RESOURCE_DESCRIPTOR # Resource list in the hardware description
%const RegType REG_RESOURCE_REQUIREMENTS_LIST 

################################################################

%newtype : RegistryOptions : DWORD : (Eq)
{{%
instance BitSet RegistryOptions where
  toWord32   = \ (RegistryOptions x) -> x
  fromWord32 = RegistryOptions
%}}

%const RegistryOptions REG_OPTION_RESERVED        # Parameter is reserved
%const RegistryOptions REG_OPTION_NON_VOLATILE    # Key is preserved when system is rebooted
%const RegistryOptions REG_OPTION_VOLATILE        # Key is not preserved when system is rebooted
%const RegistryOptions REG_OPTION_CREATE_LINK     # Created key is a symbolic link
%const RegistryOptions REG_OPTION_BACKUP_RESTORE  # open for backup or restore
			                          # special access rules privilege required
%const RegistryOptions REG_OPTION_OPEN_LINK       # Open symbolic link
%const RegistryOptions REG_LEGAL_OPTION           # (bitwise or of) all of the above

################################################################

%newtype : KeyDisposition : DWORD : (Eq)

%const KeyDisposition REG_CREATED_NEW_KEY         # New Registry Key created
%const KeyDisposition REG_OPENED_EXISTING_KEY     # Existing Key opened

################################################################
# Registry Specific Access Rights.

%newtype : REGSAM : AccessMode : (Eq)
{{%
instance BitSet REGSAM where
  toWord32   = \ (REGSAM x) -> x
  fromWord32 = REGSAM
%}}

%const REGSAM KEY_QUERY_VALUE         
%const REGSAM KEY_SET_VALUE           
%const REGSAM KEY_CREATE_SUB_KEY      
%const REGSAM KEY_ENUMERATE_SUB_KEYS  
%const REGSAM KEY_NOTIFY              
%const REGSAM KEY_CREATE_LINK         
%const REGSAM KEY_READ                # std_read  | query | enumerate_sub_keys | notify
%const REGSAM KEY_WRITE               # std_write | set | create_subkey
%const REGSAM KEY_EXECUTE             # read
%const REGSAM KEY_ALL_ACCESS          # all of the above

################################################################

%newtype : RegistryNotifyFilter : DWORD : ()
{{%
instance BitSet RegistryNotifyFilter where
  toWord32   = \ (RegistryNotifyFilter x) -> x
  fromWord32 = RegistryNotifyFilter
%}}

%const RegistryNotifyFilter REG_NOTIFY_CHANGE_NAME          # Create or delete (child)
%const RegistryNotifyFilter REG_NOTIFY_CHANGE_ATTRIBUTES    
%const RegistryNotifyFilter REG_NOTIFY_CHANGE_LAST_SET      # time stamp
%const RegistryNotifyFilter REG_NOTIFY_CHANGE_SECURITY      
%const RegistryNotifyFilter REG_LEGAL_CHANGE_FILTER         # All the above

################################################################

%errfun WinError RegCloseKey     :: HKEY -> IO ()
%errfun WinError RegConnectRegistry :: MbString -> HKEY -> IO MbHKEY using $err = RegConnectRegistry($0,$1,&$r0)
%errfun WinError RegCreateKey    :: HKEY -> String -> IO HKEY using $err = RegCreateKey($0,$1,&$r0)
%errfun WinError RegCreateKeyEx  :: HKEY -> String -> String -> RegistryOptions -> REGSAM -> MbLPSECURITY_ATTRIBUTES -> IO (HKEY, KeyDisposition) using $err = RegCreateKeyEx($0,$1,0,$2,$3,$4,$5,&$r0,&$r1)
%errfun WinError RegDeleteKey    :: HKEY -> String -> IO ()
%errfun WinError RegDeleteValue  :: HKEY -> String -> IO ()
%errfun WinError RegFlushKey     :: HKEY -> IO ()
%errfun WinError RegNotifyChangeKeyValue :: HKEY -> Bool -> RegistryNotifyFilter -> MbHANDLE -> IO () using $err = RegNotifyChangeKeyValue($0,$1,$2,$3,$3!=0)
%errfun WinError RegOpenKey      :: HKEY -> String ->           IO HKEY using $err = RegOpenKey($0,$1,&$r0)
%errfun WinError RegOpenKeyEx    :: HKEY -> String -> REGSAM -> IO HKEY using $err = RegOpenKeyEx($0,$1,0,$2,&$r0)
%errfun WinError RegQueryValueEx :: HKEY -> String -> LPBYTE -> IO (RegType, DWORD) using $err = RegQueryValueEx($0,$1,NULL,&$r0,$2,&$r1)
%errfun WinError RegSetValueEx   :: HKEY -> String -> RegType -> LPBYTE -> DWORD -> IO () using $err = RegSetValueEx($0,$1,0,$2,$3,$4)

%errfun WinError RegLoadKey      :: HKEY -> String -> String -> IO ()
%errfun WinError RegReplaceKey   :: HKEY -> String -> String -> String -> IO ()
%errfun WinError RegRestoreKey   :: HKEY -> String -> KeyRestoreFlags -> IO ()
%errfun WinError RegSaveKey      :: HKEY -> String -> LPSECURITY_ATTRIBUTES -> IO ()
%errfun WinError RegUnLoadKey    :: HKEY -> String -> IO ()

# Obsolete:
#   RegEnumKey
#   %errfun WinError RegQueryValue   :: HKEY -> String -> IO String using $err = RegOpenKeyEx($0,$1,$2,&$r0)

# ToDo:
#   %errfun WinError RegEnumKeyEx   :: HKEY -> DWORD -> IO (Maybe (String,Class,FILETIME))
#   %errfun WinError RegEnumValue   :: HKEY -> DWORD -> IO (Maybe (...))
#   %errfun WinError RegGetKeySecurity :: HKEY -> ...
#   %errfun WinError RegSetKeySecurity :: HKEY -> ...
#   %errfun WinError RegQueryInfoKey :: HKEY -> IO (String, DWORD, DWORD, DWORD, DWORD, DWORD, ..., FILETIME)
#   %errfun WinError RegQueryMultipleValues :: HKEY -> ...