/* --------------------------------------------------------------------------
 * HUGS plugin include file.
 *
 * Copyright (c) The University of Nottingham and Yale University, 1994-1997.
 * All rights reserved. See NOTICE for details and conditions of use etc...
 * Hugs version 1.4, July 1997
 *
 * $RCSfile: HUGS.h,v $
 * $Revision: 1.11 $
 * $Date: 1997/07/17 20:52:11 $
 * ------------------------------------------------------------------------*/

#define __EXTERNAL /* Avoid conflicts from within prelude.h */
#include "prelude.h"
#include "storage.h"
#include "connect.h" /* Only used to get INSTALL... */
#undef __EXTERNAL

#ifdef __GNUC__
#ifndef alloca
# define alloca __builtin_alloca
#endif
#else
# if HAVE_ALLOCA_H
#  include <alloca.h>
# else
#  ifdef _AIX
 #pragma alloca
#  else
#    ifndef alloca /* predefined by HP cc +Olibcalls */
char *alloca();
#   endif
#  endif
# endif
#endif

HugsAPI *hugs; /* pointer to virtual function table */

/* These definitions must be kept consistent with corresponding definitions
 * in storage.h
 */
#define hugs_stack(n)     (hugs->cellStack[n])
#define hugs_sp           (*(hugs->sp))
#define hugs_chkStack(n)  if (hugs_sp>=NUM_STACK-(n)) hugs->stackOverflow()
#define hugs_onto(c)	  hugs_stack(++hugs_sp)=(c)
#define hugs_push(c)      \
  do {                    \
    hugs_chkStack(1);     \
    hugs_onto(c);         \
  } while (0)
#define hugs_pushed(n)    hugs_stack(hugs_sp-(n))
#define hugs_drop()	  hugs_sp--
#define hugs_pop()	  hugs_stack(hugs_sp--)

/* These macros must yield l-values */
#define hugs_fst(c)       (*(hugs->Fst(c)))
#define hugs_snd(c)       (*(hugs->Snd(c)))
#define hugs_whnfHead     (*(hugs->whnfHead))
#define hugs_whnfInt      (*(hugs->whnfInt))
#define hugs_whnfChar     (*(hugs->whnfChar))
#define hugs_whnfFloat    (*(hugs->whnfFloat))

#define hugs_ap(x,y)      (hugs->pair(x,y))  

#define hugs_garbageCollect	hugs->garbageCollect

#define PROTO_PRIM(name)	static Void name Args((StackPtr))
#define primFun(name)		static Void name(hugs_root) StackPtr hugs_root;
#define hugs_primArg(n)		hugs_stack(hugs_root+(n))

/* IMPORTANT: the second element of an update must be written first.
 * this is to deal with the case where an INDIRECT tag is written into
 * a Cell before the second value has been set.  If a garbage collection
 * occurs before the second element was set then the INDIRECTion will be
 * (wrongly) elided and result in chaos.  I know.  It happened to me.
 */

#define hugs_update(l,r)	((hugs_snd(hugs_stack(hugs_root))=r),\
				 (hugs_fst(hugs_stack(hugs_root))=l))
#define hugs_updateRoot(c)	hugs_update(INDIRECT,c)
#define hugs_updapRoot(l,r)	hugs_update(l,r)
#define hugs_cantReduce()	hugs->evalFails(hugs_root)

typedef String CString;
typedef Float  Double;
typedef double CDouble;

#define hugsINT(x)   hugs_push(hugs->makeInt(x))
#define hugsPTR(x)   hugs_push(hugs->makePtr(x))
#define hugsCHAR(x)  hugs_push(hugs->makeChar(x))
#define hugsFLOAT(x) hugs_push(hugs->makeFloat(x))
#define hugsStablePtr(x) hugs_push(hugs->derefStablePtr(x))

#define hugsAP                                                   \
  do {                                                           \
      hugs_pushed(1) = hugs_ap(hugs_pushed(0), hugs_pushed(1));  \
      hugs_drop();                                               \
  } while (0)

#define hugsRAP                                                  \
  do {                                                           \
      hugs_pushed(1) = hugs_ap(hugs_pushed(1), hugs_pushed(0));  \
      hugs_drop();                                               \
  } while (0)

#define hugsUPDAPROOT                                            \
  do {                                                           \
      hugs_updapRoot(hugs_pushed(0), hugs_pushed(1));            \
      hugs_drop(); hugs_drop();                                  \
  } while (0)

#define hugsRUPDAPROOT                                           \
  do {                                                           \
      hugs_updapRoot(hugs_pushed(1), hugs_pushed(0));            \
      hugs_drop(); hugs_drop();                                  \
  } while (0)

/* Note: this cannot be simplified to "hugs_updateRoot(hugs_pop())" */
#define hugsUPDATEROOT                                           \
  do {                                                           \
      hugs_updateRoot(hugs_pushed(0));                           \
      hugs_drop();                                               \
  } while (0)

#define hugsTRUE     hugs_push(hugs->nameTrue)
#define hugsFALSE    hugs_push(hugs->nameFalse)

#define hugsJUST     hugs_push(hugs->nameJust)
#define hugsNOTHING  hugs_push(hugs->nameNothing);

#define hugsCONS     hugs_push(hugs->nameCons)
#define hugsNIL      hugs_push(hugs->nameNil)

#define hugsTUPLE(m) hugs_push(hugs->makeTuple(m))

#define hugsSUCC     hugs_push(hugs_primArg(1))
#define hugsFAIL     hugs_push(hugs_primArg(2))

#define hugsUNIT     hugs_push(hugs->nameUnit)

#if defined NDEBUG
#define hugs_assert(exp)  doNothing()
#else
#define hugs_assert(exp)                  \
  if (!(exp)) {                           \
      char msg[1000];                     \
      sprintf(msg,"Assertion failed: " #exp ", file " __FILE__ " line %d", __LINE__ );  \
      hugs->internal(msg);                \
  }
#endif	/* NDEBUG */
