module Xlib where
type CHAR = Char
type PString = String -- "persistent" string needs to be explicitly freed
-- Note: we do not consider NULL to be a valid pointer value.
-- Anything that requires a NULL pointer of type "T" should be
-- modified to require a "Maybe T" and an appropriate "Maybe"
-- declaration defined.
newtype Ptr = Ptr Int deriving (Eq)

unPtr :: Ptr -> Int
unPtr (Ptr i) = i

plusPtr  :: Ptr -> Int -> Ptr
plusPtr  p x = Ptr (unPtr p + x)

minusPtr :: Ptr -> Ptr -> Int
minusPtr p q = unPtr p - unPtr q
-- This is used in restricted type synonyms where there are no
-- Haskell functions which need to "look inside the representation".
-- It overcomes a (normally reasonable) restriction that the type be
-- "open" in a non-empty list of variables.
rtsDummy = error "rtsDummy"
type Word8 = Int
type Word16 = Int
type Word32 = Int
type Word = Int
type Int8 = Int
type Int16 = Int
type Int32 = Int
type Int64 = Int
type Word64 = Int
class Pointer a where
  toPtr   :: a -> Ptr
  fromPtr :: Ptr -> a
  alloc   :: IO a          -- never returns NULL ptr, may raise error.
  plus    :: a -> Int -> a
  minus   :: a -> a -> Int

free :: Pointer a => a -> IO ()
free p = freePtr (toPtr p)

cast :: (Pointer a, Pointer b) => a -> b
cast = fromPtr . toPtr

malloc :: Pointer a => Int -> IO a
malloc sz = primMalloc sz >>= \ p -> return (fromPtr p)

instance Pointer Ptr where
  toPtr   = id
  fromPtr = id
  alloc   = primMalloc 0
  plus    = plusPtr
  minus   = minusPtr
primitive isHeapExhaustion "XStest_isHeapExhaustion" :: IOError -> Bool
primitive freePtr "XS_freePtr" :: Ptr -> IO ()
primitive primMalloc "XS_primMalloc" :: Int -> IO Ptr
type MbString = Maybe String
type ListInt = [Int]
newtype XID = XID Word32 deriving ()
newtype Atom = Atom Int deriving ()
newtype VisualID = VisualID Int deriving ()
newtype Time = Time Int deriving ()
newtype Drawable = Drawable XID deriving ()
newtype Cursor = Cursor XID deriving ()
newtype Colormap = Colormap XID deriving ()
newtype KeySym = KeySym XID deriving ()
newtype Font = Font XID deriving ()
type Pixmap = Drawable
type Window = Drawable
type KeyCode = CHAR
newtype GContext = GContext XID deriving ()
--infixr 3 `andMask`
--infixr 2 `orMask`
newtype Mask = Mask Int deriving (Eq)
primitive orMask "XS_orMask" :: Mask -> Mask -> Mask
primitive andMask "XS_andMask" :: Mask -> Mask -> Mask
primitive notMask "XS_notMask" :: Mask -> Mask
emptyMask :: EventMask
emptyMask = consts_EventMask 0
type EventMask = Mask
noEventMask :: EventMask
noEventMask = consts_EventMask 1
keyPressMask :: EventMask
keyPressMask = consts_EventMask 2
keyReleaseMask :: EventMask
keyReleaseMask = consts_EventMask 3
buttonPressMask :: EventMask
buttonPressMask = consts_EventMask 4
buttonReleaseMask :: EventMask
buttonReleaseMask = consts_EventMask 5
enterWindowMask :: EventMask
enterWindowMask = consts_EventMask 6
leaveWindowMask :: EventMask
leaveWindowMask = consts_EventMask 7
pointerMotionMask :: EventMask
pointerMotionMask = consts_EventMask 8
pointerMotionHintMask :: EventMask
pointerMotionHintMask = consts_EventMask 9
button1MotionMask :: EventMask
button1MotionMask = consts_EventMask 10
button2MotionMask :: EventMask
button2MotionMask = consts_EventMask 11
button3MotionMask :: EventMask
button3MotionMask = consts_EventMask 12
button4MotionMask :: EventMask
button4MotionMask = consts_EventMask 13
button5MotionMask :: EventMask
button5MotionMask = consts_EventMask 14
buttonMotionMask :: EventMask
buttonMotionMask = consts_EventMask 15
keymapStateMask :: EventMask
keymapStateMask = consts_EventMask 16
exposureMask :: EventMask
exposureMask = consts_EventMask 17
visibilityChangeMask :: EventMask
visibilityChangeMask = consts_EventMask 18
structureNotifyMask :: EventMask
structureNotifyMask = consts_EventMask 19
resizeRedirectMask :: EventMask
resizeRedirectMask = consts_EventMask 20
substructureNotifyMask :: EventMask
substructureNotifyMask = consts_EventMask 21
substructureRedirectMask :: EventMask
substructureRedirectMask = consts_EventMask 22
focusChangeMask :: EventMask
focusChangeMask = consts_EventMask 23
propertyChangeMask :: EventMask
propertyChangeMask = consts_EventMask 24
colormapChangeMask :: EventMask
colormapChangeMask = consts_EventMask 25
ownerGrabButtonMask :: EventMask
ownerGrabButtonMask = consts_EventMask 26
newtype EventType = EventType Int deriving (Eq)
keyPress :: EventType
keyPress = consts_EventType 0
keyRelease :: EventType
keyRelease = consts_EventType 1
buttonPress :: EventType
buttonPress = consts_EventType 2
buttonRelease :: EventType
buttonRelease = consts_EventType 3
motionNotify :: EventType
motionNotify = consts_EventType 4
enterNotify :: EventType
enterNotify = consts_EventType 5
leaveNotify :: EventType
leaveNotify = consts_EventType 6
focusIn :: EventType
focusIn = consts_EventType 7
focusOut :: EventType
focusOut = consts_EventType 8
keymapNotify :: EventType
keymapNotify = consts_EventType 9
expose :: EventType
expose = consts_EventType 10
graphicsExpose :: EventType
graphicsExpose = consts_EventType 11
noExpose :: EventType
noExpose = consts_EventType 12
visibilityNotify :: EventType
visibilityNotify = consts_EventType 13
createNotify :: EventType
createNotify = consts_EventType 14
destroyNotify :: EventType
destroyNotify = consts_EventType 15
unmapNotify :: EventType
unmapNotify = consts_EventType 16
mapNotify :: EventType
mapNotify = consts_EventType 17
mapRequest :: EventType
mapRequest = consts_EventType 18
reparentNotify :: EventType
reparentNotify = consts_EventType 19
configureNotify :: EventType
configureNotify = consts_EventType 20
configureRequest :: EventType
configureRequest = consts_EventType 21
gravityNotify :: EventType
gravityNotify = consts_EventType 22
resizeRequest :: EventType
resizeRequest = consts_EventType 23
circulateNotify :: EventType
circulateNotify = consts_EventType 24
circulateRequest :: EventType
circulateRequest = consts_EventType 25
propertyNotify :: EventType
propertyNotify = consts_EventType 26
selectionClear :: EventType
selectionClear = consts_EventType 27
selectionRequest :: EventType
selectionRequest = consts_EventType 28
selectionNotify :: EventType
selectionNotify = consts_EventType 29
colormapNotify :: EventType
colormapNotify = consts_EventType 30
clientMessage :: EventType
clientMessage = consts_EventType 31
mappingNotify :: EventType
mappingNotify = consts_EventType 32
lASTEvent :: EventType
lASTEvent = consts_EventType 33
newtype Modifier = Modifier Mask deriving ()
shiftMapIndex :: Modifier
shiftMapIndex = consts_Modifier 0
lockMapIndex :: Modifier
lockMapIndex = consts_Modifier 1
controlMapIndex :: Modifier
controlMapIndex = consts_Modifier 2
mod1MapIndex :: Modifier
mod1MapIndex = consts_Modifier 3
mod2MapIndex :: Modifier
mod2MapIndex = consts_Modifier 4
mod3MapIndex :: Modifier
mod3MapIndex = consts_Modifier 5
mod4MapIndex :: Modifier
mod4MapIndex = consts_Modifier 6
mod5MapIndex :: Modifier
mod5MapIndex = consts_Modifier 7
anyModifier :: Modifier
anyModifier = consts_Modifier 8
type KeyMask = Modifier
shiftMask :: KeyMask
shiftMask = consts_KeyMask 0
lockMask :: KeyMask
lockMask = consts_KeyMask 1
controlMask :: KeyMask
controlMask = consts_KeyMask 2
mod1Mask :: KeyMask
mod1Mask = consts_KeyMask 3
mod2Mask :: KeyMask
mod2Mask = consts_KeyMask 4
mod3Mask :: KeyMask
mod3Mask = consts_KeyMask 5
mod4Mask :: KeyMask
mod4Mask = consts_KeyMask 6
mod5Mask :: KeyMask
mod5Mask = consts_KeyMask 7
type ButtonMask = Modifier
button1Mask :: ButtonMask
button1Mask = consts_ButtonMask 0
button2Mask :: ButtonMask
button2Mask = consts_ButtonMask 1
button3Mask :: ButtonMask
button3Mask = consts_ButtonMask 2
button4Mask :: ButtonMask
button4Mask = consts_ButtonMask 3
button5Mask :: ButtonMask
button5Mask = consts_ButtonMask 4
newtype Button = Button Int deriving ()
button1 :: Button
button1 = consts_Button 0
button2 :: Button
button2 = consts_Button 1
button3 :: Button
button3 = consts_Button 2
button4 :: Button
button4 = consts_Button 3
button5 :: Button
button5 = consts_Button 4
newtype NotifyMode = NotifyMode Int deriving (Eq)
notifyNormal :: NotifyMode
notifyNormal = consts_NotifyMode 0
notifyGrab :: NotifyMode
notifyGrab = consts_NotifyMode 1
notifyUngrab :: NotifyMode
notifyUngrab = consts_NotifyMode 2
notifyWhileGrabbed :: NotifyMode
notifyWhileGrabbed = consts_NotifyMode 3
notifyHint :: NotifyMode
notifyHint = consts_NotifyMode 4
newtype NotifyDetail = NotifyDetail Int deriving ()
notifyAncestor :: NotifyDetail
notifyAncestor = consts_NotifyDetail 0
notifyVirtual :: NotifyDetail
notifyVirtual = consts_NotifyDetail 1
notifyInferior :: NotifyDetail
notifyInferior = consts_NotifyDetail 2
notifyNonlinear :: NotifyDetail
notifyNonlinear = consts_NotifyDetail 3
notifyNonlinearVirtual :: NotifyDetail
notifyNonlinearVirtual = consts_NotifyDetail 4
notifyPointer :: NotifyDetail
notifyPointer = consts_NotifyDetail 5
notifyPointerRoot :: NotifyDetail
notifyPointerRoot = consts_NotifyDetail 6
notifyDetailNone :: NotifyDetail
notifyDetailNone = consts_NotifyDetail 7
newtype Visibility = Visibility Int deriving ()
visibilityUnobscured :: Visibility
visibilityUnobscured = consts_Visibility 0
visibilityPartiallyObscured :: Visibility
visibilityPartiallyObscured = consts_Visibility 1
visibilityFullyObscured :: Visibility
visibilityFullyObscured = consts_Visibility 2
newtype Place = Place Int deriving ()
placeOnTop :: Place
placeOnTop = consts_Place 0
placeOnBottom :: Place
placeOnBottom = consts_Place 1
newtype Protocol = Protocol Int deriving ()
familyInternet :: Protocol
familyInternet = consts_Protocol 0
familyDECnet :: Protocol
familyDECnet = consts_Protocol 1
familyChaos :: Protocol
familyChaos = consts_Protocol 2
newtype PropertyNotification = PropertyNotification Int deriving ()
propertyNewValue :: PropertyNotification
propertyNewValue = consts_PropertyNotification 0
propertyDelete :: PropertyNotification
propertyDelete = consts_PropertyNotification 1
newtype ColormapNotification = ColormapNotification Int deriving ()
colormapUninstalled :: ColormapNotification
colormapUninstalled = consts_ColormapNotification 0
colormapInstalled :: ColormapNotification
colormapInstalled = consts_ColormapNotification 1
newtype GrabMode = GrabMode Int deriving ()
grabModeSync :: GrabMode
grabModeSync = consts_GrabMode 0
grabModeAsync :: GrabMode
grabModeAsync = consts_GrabMode 1
newtype GrabStatus = GrabStatus Int deriving ()
grabSuccess :: GrabStatus
grabSuccess = consts_GrabStatus 0
alreadyGrabbed :: GrabStatus
alreadyGrabbed = consts_GrabStatus 1
grabInvalidTime :: GrabStatus
grabInvalidTime = consts_GrabStatus 2
grabNotViewable :: GrabStatus
grabNotViewable = consts_GrabStatus 3
grabFrozen :: GrabStatus
grabFrozen = consts_GrabStatus 4
newtype AllowEvents = AllowEvents Int deriving ()
asyncPointer :: AllowEvents
asyncPointer = consts_AllowEvents 0
syncPointer :: AllowEvents
syncPointer = consts_AllowEvents 1
replayPointer :: AllowEvents
replayPointer = consts_AllowEvents 2
asyncKeyboard :: AllowEvents
asyncKeyboard = consts_AllowEvents 3
syncKeyboard :: AllowEvents
syncKeyboard = consts_AllowEvents 4
replayKeyboard :: AllowEvents
replayKeyboard = consts_AllowEvents 5
asyncBoth :: AllowEvents
asyncBoth = consts_AllowEvents 6
syncBoth :: AllowEvents
syncBoth = consts_AllowEvents 7
newtype FocusMode = FocusMode Int deriving ()
revertToNone :: FocusMode
revertToNone = consts_FocusMode 0
revertToPointerRoot :: FocusMode
revertToPointerRoot = consts_FocusMode 1
revertToParent :: FocusMode
revertToParent = consts_FocusMode 2
newtype Status = Status Int deriving ()
success :: Status
success = consts_Status 0
badRequest :: Status
badRequest = consts_Status 1
badValue :: Status
badValue = consts_Status 2
badWindow :: Status
badWindow = consts_Status 3
badPixmap :: Status
badPixmap = consts_Status 4
badAtom :: Status
badAtom = consts_Status 5
badCursor :: Status
badCursor = consts_Status 6
badFont :: Status
badFont = consts_Status 7
badMatch :: Status
badMatch = consts_Status 8
badDrawable :: Status
badDrawable = consts_Status 9
badAccess :: Status
badAccess = consts_Status 10
badAlloc :: Status
badAlloc = consts_Status 11
badColor :: Status
badColor = consts_Status 12
badGC :: Status
badGC = consts_Status 13
badIDChoice :: Status
badIDChoice = consts_Status 14
badName :: Status
badName = consts_Status 15
badLength :: Status
badLength = consts_Status 16
badImplementation :: Status
badImplementation = consts_Status 17
firstExtensionError :: Status
firstExtensionError = consts_Status 18
lastExtensionError :: Status
lastExtensionError = consts_Status 19
newtype WindowClass = WindowClass Int deriving ()
copyFromParent :: WindowClass
copyFromParent = consts_WindowClass 0
inputOutput :: WindowClass
inputOutput = consts_WindowClass 1
inputOnly :: WindowClass
inputOnly = consts_WindowClass 2
type AttributeMask = Mask
cWBackPixmap :: AttributeMask
cWBackPixmap = consts_AttributeMask 0
cWBackPixel :: AttributeMask
cWBackPixel = consts_AttributeMask 1
cWBorderPixmap :: AttributeMask
cWBorderPixmap = consts_AttributeMask 2
cWBorderPixel :: AttributeMask
cWBorderPixel = consts_AttributeMask 3
cWBitGravity :: AttributeMask
cWBitGravity = consts_AttributeMask 4
cWWinGravity :: AttributeMask
cWWinGravity = consts_AttributeMask 5
cWBackingStore :: AttributeMask
cWBackingStore = consts_AttributeMask 6
cWBackingPlanes :: AttributeMask
cWBackingPlanes = consts_AttributeMask 7
cWBackingPixel :: AttributeMask
cWBackingPixel = consts_AttributeMask 8
cWOverrideRedirect :: AttributeMask
cWOverrideRedirect = consts_AttributeMask 9
cWSaveUnder :: AttributeMask
cWSaveUnder = consts_AttributeMask 10
cWEventMask :: AttributeMask
cWEventMask = consts_AttributeMask 11
cWDontPropagate :: AttributeMask
cWDontPropagate = consts_AttributeMask 12
cWColormap :: AttributeMask
cWColormap = consts_AttributeMask 13
cWCursor :: AttributeMask
cWCursor = consts_AttributeMask 14
newtype Gravity = Gravity Int deriving (Eq)
type BitGravity = Gravity
type WindowGravity = Gravity
unmapGravity :: WindowGravity
unmapGravity = consts_WindowGravity 0
forgetGravity :: BitGravity
forgetGravity = consts_BitGravity 0
northWestGravity :: Gravity
northWestGravity = consts_Gravity 0
northGravity :: Gravity
northGravity = consts_Gravity 1
northEastGravity :: Gravity
northEastGravity = consts_Gravity 2
westGravity :: Gravity
westGravity = consts_Gravity 3
centerGravity :: Gravity
centerGravity = consts_Gravity 4
eastGravity :: Gravity
eastGravity = consts_Gravity 5
southWestGravity :: Gravity
southWestGravity = consts_Gravity 6
southGravity :: Gravity
southGravity = consts_Gravity 7
southEastGravity :: Gravity
southEastGravity = consts_Gravity 8
staticGravity :: Gravity
staticGravity = consts_Gravity 9
newtype BackingStore = BackingStore Int deriving (Eq)
notUseful :: BackingStore
notUseful = consts_BackingStore 0
whenMapped :: BackingStore
whenMapped = consts_BackingStore 1
always :: BackingStore
always = consts_BackingStore 2
newtype CoordinateMode = CoordinateMode Int deriving (Eq)
coordModeOrigin :: CoordinateMode
coordModeOrigin = consts_CoordinateMode 0
coordModePrevious :: CoordinateMode
coordModePrevious = consts_CoordinateMode 1
newtype PolygonShape = PolygonShape Int deriving (Eq)
complex :: PolygonShape
complex = consts_PolygonShape 0
nonconvex :: PolygonShape
nonconvex = consts_PolygonShape 1
convex :: PolygonShape
convex = consts_PolygonShape 2
newtype QueryBestSizeClass = QueryBestSizeClass Int deriving (Eq)
cursorShape :: QueryBestSizeClass
cursorShape = consts_QueryBestSizeClass 0
tileShape :: QueryBestSizeClass
tileShape = consts_QueryBestSizeClass 1
stippleShape :: QueryBestSizeClass
stippleShape = consts_QueryBestSizeClass 2
newtype ColormapAlloc = ColormapAlloc Int deriving (Eq)
allocNone :: ColormapAlloc
allocNone = consts_ColormapAlloc 0
allocAll :: ColormapAlloc
allocAll = consts_ColormapAlloc 1
newtype Access = Access Int deriving (Eq)
enableAccess :: Access
enableAccess = consts_Access 0
disableAccess :: Access
disableAccess = consts_Access 1
newtype CloseDownMode = CloseDownMode Int deriving (Eq)
destroyAll :: CloseDownMode
destroyAll = consts_CloseDownMode 0
retainPermanent :: CloseDownMode
retainPermanent = consts_CloseDownMode 1
retainTemporary :: CloseDownMode
retainTemporary = consts_CloseDownMode 2
newtype ChangeSaveSetMode = ChangeSaveSetMode Int deriving (Eq)
setModeInsert :: ChangeSaveSetMode
setModeInsert = consts_ChangeSaveSetMode 0
setModeDelete :: ChangeSaveSetMode
setModeDelete = consts_ChangeSaveSetMode 1
newtype WindowStackingMethod = WindowStackingMethod Int deriving (Eq)
above :: WindowStackingMethod
above = consts_WindowStackingMethod 0
below :: WindowStackingMethod
below = consts_WindowStackingMethod 1
topIf :: WindowStackingMethod
topIf = consts_WindowStackingMethod 2
bottomIf :: WindowStackingMethod
bottomIf = consts_WindowStackingMethod 3
opposite :: WindowStackingMethod
opposite = consts_WindowStackingMethod 4
newtype CirculationDirection = CirculationDirection Int deriving (Eq)
raiseLowest :: CirculationDirection
raiseLowest = consts_CirculationDirection 0
lowerHighest :: CirculationDirection
lowerHighest = consts_CirculationDirection 1
newtype RectangleOrdering = RectangleOrdering Int deriving (Eq)
unsorted :: RectangleOrdering
unsorted = consts_RectangleOrdering 0
ySorted :: RectangleOrdering
ySorted = consts_RectangleOrdering 1
yXSorted :: RectangleOrdering
yXSorted = consts_RectangleOrdering 2
yXBanded :: RectangleOrdering
yXBanded = consts_RectangleOrdering 3
newtype ImageFormat = ImageFormat Int deriving (Eq)
xYBitmap :: ImageFormat
xYBitmap = consts_ImageFormat 0
xYPixmap :: ImageFormat
xYPixmap = consts_ImageFormat 1
zPixmap :: ImageFormat
zPixmap = consts_ImageFormat 2
newtype AutoRepeatMode = AutoRepeatMode Int deriving (Eq)
autoRepeatModeOff :: AutoRepeatMode
autoRepeatModeOff = consts_AutoRepeatMode 0
autoRepeatModeOn :: AutoRepeatMode
autoRepeatModeOn = consts_AutoRepeatMode 1
autoRepeatModeDefault :: AutoRepeatMode
autoRepeatModeDefault = consts_AutoRepeatMode 2
newtype LedMode = LedMode Int deriving (Eq)
ledModeOff :: LedMode
ledModeOff = consts_LedMode 0
ledModeOn :: LedMode
ledModeOn = consts_LedMode 1
newtype ByteOrder = ByteOrder Int deriving (Eq)
lSBFirst :: ByteOrder
lSBFirst = consts_ByteOrder 0
mSBFirst :: ByteOrder
mSBFirst = consts_ByteOrder 1
newtype VisualClass = VisualClass Int deriving (Eq)
staticGray :: VisualClass
staticGray = consts_VisualClass 0
grayScale :: VisualClass
grayScale = consts_VisualClass 1
staticColor :: VisualClass
staticColor = consts_VisualClass 2
pseudoColor :: VisualClass
pseudoColor = consts_VisualClass 3
trueColor :: VisualClass
trueColor = consts_VisualClass 4
directColor :: VisualClass
directColor = consts_VisualClass 5
newtype MappingRequest = MappingRequest Int deriving (Eq)
mappingModifier :: MappingRequest
mappingModifier = consts_MappingRequest 0
mappingKeyboard :: MappingRequest
mappingKeyboard = consts_MappingRequest 1
mappingPointer :: MappingRequest
mappingPointer = consts_MappingRequest 2
newtype PrimaryMask = PrimaryMask Mask deriving (Eq)
doRed :: PrimaryMask
doRed = consts_PrimaryMask 0
doGreen :: PrimaryMask
doGreen = consts_PrimaryMask 1
doBlue :: PrimaryMask
doBlue = consts_PrimaryMask 2
{-

#define MappingSuccess     	0
#define MappingBusy        	1
#define MappingFailed		2


/* ConfigureWindow structure */

#define CWX			(1<<0)
#define CWY			(1<<1)
#define CWWidth			(1<<2)
#define CWHeight		(1<<3)
#define CWBorderWidth		(1<<4)
#define CWSibling		(1<<5)
#define CWStackMode		(1<<6)

/* Used in GetWindowAttributes reply */

#define IsUnmapped		0
#define IsUnviewable		1
#define IsViewable		2

/* Property modes */

#define PropModeReplace         0
#define PropModePrepend         1
#define PropModeAppend          2

/* GC components: masks used in CreateGC, CopyGC, ChangeGC, OR'ed into
   GC.stateChanges */

#define GCFunction              (1L<<0)
#define GCPlaneMask             (1L<<1)
#define GCForeground            (1L<<2)
#define GCBackground            (1L<<3)
#define GCLineWidth             (1L<<4)
#define GCLineStyle             (1L<<5)
#define GCCapStyle              (1L<<6)
#define GCJoinStyle		(1L<<7)
#define GCFillStyle		(1L<<8)
#define GCFillRule		(1L<<9)
#define GCTile			(1L<<10)
#define GCStipple		(1L<<11)
#define GCTileStipXOrigin	(1L<<12)
#define GCTileStipYOrigin	(1L<<13)
#define GCFont 			(1L<<14)
#define GCSubwindowMode		(1L<<15)
#define GCGraphicsExposures     (1L<<16)
#define GCClipXOrigin		(1L<<17)
#define GCClipYOrigin		(1L<<18)
#define GCClipMask		(1L<<19)
#define GCDashOffset		(1L<<20)
#define GCDashList		(1L<<21)
#define GCArcMode		(1L<<22)

#define GCLastBit		22





/* masks for ChangeKeyboardControl */

#define KBKeyClickPercent	(1L<<0)
#define KBBellPercent		(1L<<1)
#define KBBellPitch		(1L<<2)
#define KBBellDuration		(1L<<3)
#define KBLed			(1L<<4)
#define KBLedMode		(1L<<5)
#define KBKey			(1L<<6)
#define KBAutoRepeatMode	(1L<<7)


/*****************************************************************
 * FONTS
 *****************************************************************/

/* used in QueryFont -- draw direction */

#define FontLeftToRight		0
#define FontRightToLeft		1

#define FontChange		255

/*****************************************************************
 * HOSTS AND CONNECTIONS
 *****************************************************************/

/* for ChangeHosts */

#define HostInsert		0
#define HostDelete		1



#endif /* X_H */
-}
type Pixel = Word32
type Position = Int32
type Dimension = Word
type ScreenNumber = Word32
type Byte = CHAR
type Buffer = Int
type Display    = Ptr in rtsDummy
type Screen      = Ptr in rtsDummy
type Visual      = Ptr in rtsDummy
type XFontStruct = Ptr in rtsDummy
newtype GC = GC Ptr deriving ()
type XPoint = (Position,Position)
type ListXPoint = [XPoint]
type XSegment = (Position,Position,Position,Position)
type ListXSegment = [XSegment]
type XRectangle = (Position,Position,Dimension,Dimension)
type ListXRectangle = [XRectangle]
type XArc = (Position,Position,Dimension,Dimension,Int,Int)
type ListXArc = [XArc]
type ListWindow = [Window]
type ListColormap = [Colormap]
type ListPixel = [Pixel]
type XColor = (Pixel,Word8,Word8,Word8,Word8)
type ListXColor = [XColor]
newtype Shape = Shape Int deriving ()
primitive isNullPtr "XStest_isNullPtr" :: IOError -> Bool
primitive isBadStatus "XStest_isBadStatus" :: IOError -> Maybe (Status )
primitive isZero "XStest_isZero" :: IOError -> Bool
primitive isFalse "XStest_isFalse" :: IOError -> Bool
primitive xAllPlanes "XS_XAllPlanes" :: Pixel
primitive xBlackPixel "XS_XBlackPixel" :: Display -> ScreenNumber -> Pixel
primitive xWhitePixel "XS_XWhitePixel" :: Display -> ScreenNumber -> Pixel
primitive connectionNumber "XS_ConnectionNumber" :: Display -> Int
primitive xDefaultColormap "XS_XDefaultColormap" :: Display -> ScreenNumber -> Colormap
primitive xListDepths "XS_XListDepths" :: Display -> ScreenNumber -> ListInt
primitive xDefaultGC "XS_XDefaultGC" :: Display -> ScreenNumber -> GC
primitive xDefaultDepth "XS_XDefaultDepth" :: Display -> ScreenNumber -> Int
primitive xDefaultScreen "XS_XDefaultScreen" :: Display -> ScreenNumber
primitive xDefaultScreenOfDisplay "XS_XDefaultScreenOfDisplay" :: Display -> Screen
primitive displayHeight "XS_DisplayHeight" :: Display -> ScreenNumber -> Int
primitive displayHeightMM "XS_DisplayHeightMM" :: Display -> ScreenNumber -> Int
primitive displayWidth "XS_DisplayWidth" :: Display -> ScreenNumber -> Int
primitive displayWidthMM "XS_DisplayWidthMM" :: Display -> ScreenNumber -> Int
primitive xMaxRequestSize "XS_XMaxRequestSize" :: Display -> Int
primitive xDisplayMotionBufferSize "XS_XDisplayMotionBufferSize" :: Display -> Int
primitive xResourceManagerString "XS_XResourceManagerString" :: Display -> String
primitive xScreenResourceString "XS_XScreenResourceString" :: Screen   -> String
primitive displayString "XS_DisplayString" :: Display -> String
primitive xImageByteOrder "XS_XImageByteOrder" :: Display -> Int
primitive xProtocolRevision "XS_XProtocolRevision" :: Display -> Int
primitive xProtocolVersion "XS_XProtocolVersion" :: Display -> Int
primitive xServerVendor "XS_XServerVendor" :: Display -> String
primitive xScreenCount "XS_XScreenCount" :: Display -> Int
primitive xDefaultVisual "XS_XDefaultVisual" :: Display -> ScreenNumber -> Visual
primitive displayCells "XS_DisplayCells" :: Display -> ScreenNumber -> Int
primitive displayPlanes "XS_DisplayPlanes" :: Display -> ScreenNumber -> Int
primitive xScreenOfDisplay "XS_XScreenOfDisplay" :: Display -> ScreenNumber -> Screen
primitive xDefaultRootWindow "XS_XDefaultRootWindow" :: Display -> Window
primitive xRootWindow "XS_XRootWindow" :: Display -> ScreenNumber -> IO Window
primitive xQLength "XS_XQLength" :: Display -> IO Int
primitive xNoOp "XS_XNoOp" :: Display -> IO ()
primitive xOpenDisplay "XS_XOpenDisplay" :: String -> IO Display
primitive xCloseDisplay "XS_XCloseDisplay" :: Display -> IO ()
primitive xrmInitialize "XS_XrmInitialize" :: IO ()
primitive xAutoRepeatOff "XS_XAutoRepeatOff" :: Display -> IO ()
primitive xAutoRepeatOn "XS_XAutoRepeatOn" :: Display -> IO ()
primitive xBell "XS_XBell" :: Display -> Int -> IO ()
primitive xSetCloseDownMode "XS_XSetCloseDownMode" :: Display -> CloseDownMode -> IO ()
primitive xLastKnownRequestProcessed "XS_XLastKnownRequestProcessed" :: Display -> IO Int
primitive xGetInputFocus "XS_XGetInputFocus" :: Display -> IO (Window, FocusMode)
primitive xSetInputFocus "XS_XSetInputFocus" :: Display -> Window -> FocusMode -> Time -> IO ()
primitive xGrabButton "XS_XGrabButton" :: Display -> Button -> ButtonMask -> Window -> Bool -> EventMask -> GrabMode -> GrabMode -> Window -> Cursor -> IO ()
primitive xUngrabButton "XS_XUngrabButton" :: Display -> Button -> ButtonMask -> Window -> IO ()
primitive xGrabPointer "XS_XGrabPointer" :: Display -> Window -> Bool -> EventMask -> GrabMode -> GrabMode -> Window -> Cursor -> Time -> IO GrabStatus
primitive xUngrabPointer "XS_XUngrabPointer" :: Display -> Time -> IO ()
primitive xGrabKey "XS_XGrabKey" :: Display -> KeyCode -> ButtonMask -> Window -> Bool -> GrabMode -> GrabMode -> IO ()
primitive xUngrabKey "XS_XUngrabKey" :: Display -> KeyCode -> ButtonMask -> Window -> IO ()
primitive xGrabKeyboard "XS_XGrabKeyboard" :: Display -> Window -> Bool -> GrabMode -> GrabMode -> Time -> IO GrabStatus
primitive xUngrabKeyboard "XS_XUngrabKeyboard" :: Display -> Time -> IO ()
primitive xGrabServer "XS_XGrabServer" :: Display -> IO ()
primitive xUngrabServer "XS_XUngrabServer" :: Display -> IO ()
primitive xQueryBestTile "XS_XQueryBestTile" :: Display -> Drawable ->                       Dimension -> Dimension -> IO (Dimension, Dimension) 
primitive xQueryBestStipple "XS_XQueryBestStipple" :: Display -> Drawable ->                       Dimension -> Dimension -> IO (Dimension, Dimension) 
primitive xQueryBestCursor "XS_XQueryBestCursor" :: Display -> Drawable ->                       Dimension -> Dimension -> IO (Dimension, Dimension) 
primitive xQueryBestSize "XS_XQueryBestSize" :: Display -> Drawable -> QueryBestSizeClass -> Dimension -> Dimension -> IO (Dimension, Dimension) 
primitive xQueryPointer "XS_XQueryPointer" :: Display -> Window -> IO (Bool, Window, Window, Int, Int, Int, Int, Modifier)
primitive xDisplayName "XS_XDisplayName" :: String -> String
primitive xEnableAccessControl "XS_XEnableAccessControl" :: Display -> IO ()
primitive xDisableAccessControl "XS_XDisableAccessControl" :: Display -> IO ()
primitive xSetAccessControl "XS_XSetAccessControl" :: Display -> Access -> IO ()
newtype QueuedMode = QueuedMode Int deriving (Eq)
queuedAlready :: QueuedMode
queuedAlready = consts_QueuedMode 0
queuedAfterFlush :: QueuedMode
queuedAfterFlush = consts_QueuedMode 1
queuedAfterReading :: QueuedMode
queuedAfterReading = consts_QueuedMode 2
type XEvent = (EventType)
type XEventPtr = Ptr in
  toXEventPtr   :: Ptr -> XEventPtr,
  fromXEventPtr :: XEventPtr -> Ptr

toXEventPtr   x = x 
fromXEventPtr x = x

instance Pointer XEventPtr where
  toPtr     = fromXEventPtr
  fromPtr   = toXEventPtr
  alloc     = malloc sizeofXEventPtr
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofXEventPtr))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofXEventPtr

primitive getXEventPtr        "XS_get_XEventPtr"    :: XEventPtr -> IO XEvent
primitive setXEventPtr        "XS_set_XEventPtr"    :: XEventPtr -> XEvent -> IO ()
sizeofXEventPtr :: Int
sizeofXEventPtr = consts_Int 0
type XAnyEvent = (Int32,Bool,Display,Window)
type XKeyEvent = (Window,Window,Time,Int,Int,Int,Int,Modifier,KeyCode,Bool)
type XKeyEventPtr = Ptr in
  toXKeyEventPtr   :: Ptr -> XKeyEventPtr,
  fromXKeyEventPtr :: XKeyEventPtr -> Ptr

toXKeyEventPtr   x = x 
fromXKeyEventPtr x = x

instance Pointer XKeyEventPtr where
  toPtr     = fromXKeyEventPtr
  fromPtr   = toXKeyEventPtr
  alloc     = malloc sizeofXKeyEventPtr
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofXKeyEventPtr))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofXKeyEventPtr

primitive getXKeyEventPtr        "XS_get_XKeyEventPtr"    :: XKeyEventPtr -> IO XKeyEvent
primitive setXKeyEventPtr        "XS_set_XKeyEventPtr"    :: XKeyEventPtr -> XKeyEvent -> IO ()
sizeofXKeyEventPtr :: Int
sizeofXKeyEventPtr = consts_Int 1
type XButtonEvent = (Window,Window,Time,Int,Int,Int,Int,Modifier,Button,Bool)
type XMotionEvent = (Window,Window,Time,Int,Int,Int,Int,Modifier,NotifyMode,Bool)
type XCrossingEvent = (Window,Window,Time,Int,Int,Int,Int,NotifyMode,NotifyDetail,Bool,Bool,Modifier)
type XFocusChangeEvent = (NotifyMode,NotifyDetail)
type XExposeEvent = (Position,Position,Dimension,Dimension,Int)
type XGraphicsExposeEvent = (Position,Position,Dimension,Dimension,Int,Int,Int)
type XCirculateEvent = (Window,Place)
type XConfigureEvent = (Window,Position,Position,Dimension,Dimension,Dimension,Window,Bool)
type XCreateWindowEvent = (Window,Position,Position,Dimension,Dimension,Dimension,Bool)
type XDestroyWindowEvent = (Window)
type XGravityEvent = (Window,Position,Position)
type XMapEvent = (Bool)
type XMappingEvent = (MappingRequest,KeyCode,Int)
type XReparentEvent = (Window,Window,Position,Position,Bool)
type XUnmapEvent = (Window,Bool)
type XVisibilityEvent = (Visibility)
type XCirculateRequestEvent = (Place)
type XMapRequestEvent = (Window)
type XResizeRequestEvent = (Dimension,Dimension)
type XColormapEvent = (Colormap,Bool,ColormapNotification)
primitive xFlush "XS_XFlush" :: Display ->               IO ()
primitive xSync "XS_XSync" :: Display -> Bool ->       IO ()
primitive xPending "XS_XPending" :: Display ->               IO Int
primitive xEventsQueued "XS_XEventsQueued" :: Display -> QueuedMode -> IO Int
primitive xNextEvent "XS_XNextEvent" :: Display -> XEventPtr  -> IO ()
primitive xAllowEvents "XS_XAllowEvents" :: Display -> AllowEvents -> Time -> IO ()
primitive xSelectInput "XS_XSelectInput" :: Display -> Window -> EventMask -> IO ()
primitive xSendEvent "XS_XSendEvent" :: Display -> Window -> Bool -> EventMask -> XEventPtr -> IO ()
primitive xWindowEvent "XS_XWindowEvent" :: Display -> Window -> EventMask -> XEventPtr -> IO ()
primitive xCheckWindowEvent "XS_XCheckWindowEvent" :: Display -> Window -> EventMask -> XEventPtr -> IO Bool
primitive xMaskEvent "XS_XMaskEvent" :: Display -> EventMask -> XEventPtr -> IO ()
primitive xCheckMaskEvent "XS_XCheckMaskEvent" :: Display -> EventMask -> XEventPtr -> IO Bool
primitive xCheckTypedEvent "XS_XCheckTypedEvent" :: Display -> EventType -> XEventPtr -> IO Bool
primitive xCheckTypedWindowEvent "XS_XCheckTypedWindowEvent" :: Display -> Window -> EventType -> XEventPtr -> IO Bool
primitive xPutBackEvent "XS_XPutBackEvent" :: Display -> XEventPtr -> IO ()
primitive xPeekEvent "XS_XPeekEvent" :: Display -> XEventPtr -> IO ()
primitive xRefreshKeyboardMapping "XS_XRefreshKeyboardMapping" :: XMappingEvent -> IO ()
primitive xGeometry "XS_XGeometry" :: Display -> Int -> String -> String -> Dimension -> Dimension -> Dimension -> Int -> Int -> IO (Int, Position, Position, Dimension, Dimension)
primitive xGetGeometry "XS_XGetGeometry" :: Display -> Int -> Drawable -> IO (Window, Position, Position, Dimension, Dimension, Dimension, Int) 
primitive xSupportsLocale "XS_XSupportsLocale" :: IO Bool
primitive xSetLocaleModifiers "XS_XSetLocaleModifiers" :: String -> IO String
newtype AllowExposuresMode = AllowExposuresMode Int deriving (Eq)
dontAllowExposures :: AllowExposuresMode
dontAllowExposures = consts_AllowExposuresMode 0
allowExposures :: AllowExposuresMode
allowExposures = consts_AllowExposuresMode 1
defaultExposures :: AllowExposuresMode
defaultExposures = consts_AllowExposuresMode 2
newtype PreferBlankingMode = PreferBlankingMode Int deriving (Eq)
dontPreferBlanking :: PreferBlankingMode
dontPreferBlanking = consts_PreferBlankingMode 0
preferBlanking :: PreferBlankingMode
preferBlanking = consts_PreferBlankingMode 1
defaultBlanking :: PreferBlankingMode
defaultBlanking = consts_PreferBlankingMode 2
newtype ScreenSaverMode = ScreenSaverMode Int deriving (Eq)
screenSaverActive :: ScreenSaverMode
screenSaverActive = consts_ScreenSaverMode 0
screenSaverReset :: ScreenSaverMode
screenSaverReset = consts_ScreenSaverMode 1
primitive xGetScreenSaver "XS_XGetScreenSaver" :: Display -> IO (Int, Int, PreferBlankingMode, AllowExposuresMode)
primitive xSetScreenSaver "XS_XSetScreenSaver" :: Display -> Int -> Int -> PreferBlankingMode -> AllowExposuresMode -> IO ()
primitive xActivateScreenSaver "XS_XActivateScreenSaver" :: Display -> IO ()
primitive xResetScreenSaver "XS_XResetScreenSaver" :: Display -> IO ()
primitive xForceScreenSaver "XS_XForceScreenSaver" :: Display -> ScreenSaverMode -> IO ()
primitive xGetPointerControl "XS_XGetPointerControl" :: Display -> IO (Int, Int, Int)
primitive xWarpPointer "XS_XWarpPointer" :: Display -> Window -> Window -> Position -> Position -> Dimension -> Dimension -> Position -> Position -> IO ()
primitive xBlackPixelOfScreen "XS_XBlackPixelOfScreen" :: Screen -> Pixel
primitive xWhitePixelOfScreen "XS_XWhitePixelOfScreen" :: Screen -> Pixel
primitive xCellsOfScreen "XS_XCellsOfScreen" :: Screen -> Int
primitive xDefaultColormapOfScreen "XS_XDefaultColormapOfScreen" :: Screen -> Colormap
primitive xDefaultDepthOfScreen "XS_XDefaultDepthOfScreen" :: Screen -> Int
primitive xDefaultGCOfScreen "XS_XDefaultGCOfScreen" :: Screen -> GC
primitive xDefaultVisualOfScreen "XS_XDefaultVisualOfScreen" :: Screen -> Visual
primitive xDoesBackingStore "XS_XDoesBackingStore" :: Screen -> Bool
primitive xDoesSaveUnders "XS_XDoesSaveUnders" :: Screen -> Bool
primitive displayOfScreen "XS_DisplayOfScreen" :: Screen -> Display
primitive xEventMaskOfScreen "XS_XEventMaskOfScreen" :: Screen -> EventMask
primitive xMinCmapsOfScreen "XS_XMinCmapsOfScreen" :: Screen -> Int
primitive xMaxCmapsOfScreen "XS_XMaxCmapsOfScreen" :: Screen -> Int
primitive xRootWindowOfScreen "XS_XRootWindowOfScreen" :: Screen -> Window
primitive xWidthOfScreen "XS_XWidthOfScreen" :: Screen -> Dimension
primitive xWidthMMOfScreen "XS_XWidthMMOfScreen" :: Screen -> Dimension
primitive xHeightOfScreen "XS_XHeightOfScreen" :: Screen -> Dimension
primitive xHeightMMOfScreen "XS_XHeightMMOfScreen" :: Screen -> Dimension
primitive xPlanesOfScreen "XS_XPlanesOfScreen" :: Screen -> Int
primitive xScreenNumberOfScreen "XS_XScreenNumberOfScreen" :: Screen -> ScreenNumber
newtype ArcMode = ArcMode Int deriving (Eq)
arcChord :: ArcMode
arcChord = consts_ArcMode 0
arcPieSlice :: ArcMode
arcPieSlice = consts_ArcMode 1
newtype SubWindowMode = SubWindowMode Int deriving (Eq)
clipByChildren :: SubWindowMode
clipByChildren = consts_SubWindowMode 0
includeInferiors :: SubWindowMode
includeInferiors = consts_SubWindowMode 1
newtype LineStyle = LineStyle Int deriving (Eq)
lineSolid :: LineStyle
lineSolid = consts_LineStyle 0
lineDoubleDash :: LineStyle
lineDoubleDash = consts_LineStyle 1
lineOnOffDash :: LineStyle
lineOnOffDash = consts_LineStyle 2
newtype CapStyle = CapStyle Int deriving (Eq)
capNotLast :: CapStyle
capNotLast = consts_CapStyle 0
capButt :: CapStyle
capButt = consts_CapStyle 1
capRound :: CapStyle
capRound = consts_CapStyle 2
capProjecting :: CapStyle
capProjecting = consts_CapStyle 3
newtype JoinStyle = JoinStyle Int deriving (Eq)
joinMiter :: JoinStyle
joinMiter = consts_JoinStyle 0
joinRound :: JoinStyle
joinRound = consts_JoinStyle 1
joinBevel :: JoinStyle
joinBevel = consts_JoinStyle 2
newtype FillStyle = FillStyle Int deriving (Eq)
fillSolid :: FillStyle
fillSolid = consts_FillStyle 0
fillTiled :: FillStyle
fillTiled = consts_FillStyle 1
fillOpaqueStippled :: FillStyle
fillOpaqueStippled = consts_FillStyle 2
fillStippled :: FillStyle
fillStippled = consts_FillStyle 3
newtype FillRule = FillRule Int deriving (Eq)
evenOddRule :: FillRule
evenOddRule = consts_FillRule 0
windingRule :: FillRule
windingRule = consts_FillRule 1
newtype GXFunction = GXFunction Int deriving (Eq)
gXclear :: GXFunction
gXclear = consts_GXFunction 0
gXand :: GXFunction
gXand = consts_GXFunction 1
gXandReverse :: GXFunction
gXandReverse = consts_GXFunction 2
gXcopy :: GXFunction
gXcopy = consts_GXFunction 3
gXandInverted :: GXFunction
gXandInverted = consts_GXFunction 4
gXnoop :: GXFunction
gXnoop = consts_GXFunction 5
gXxor :: GXFunction
gXxor = consts_GXFunction 6
gXor :: GXFunction
gXor = consts_GXFunction 7
gXnor :: GXFunction
gXnor = consts_GXFunction 8
gXequiv :: GXFunction
gXequiv = consts_GXFunction 9
gXinvert :: GXFunction
gXinvert = consts_GXFunction 10
gXorReverse :: GXFunction
gXorReverse = consts_GXFunction 11
gXcopyInverted :: GXFunction
gXcopyInverted = consts_GXFunction 12
gXorInverted :: GXFunction
gXorInverted = consts_GXFunction 13
gXnand :: GXFunction
gXnand = consts_GXFunction 14
gXset :: GXFunction
gXset = consts_GXFunction 15
primitive xSetArcMode "XS_XSetArcMode" :: Display -> GC -> ArcMode        	      -> IO ()
primitive xSetBackground "XS_XSetBackground" :: Display -> GC -> Pixel          	      -> IO ()
primitive xSetForeground "XS_XSetForeground" :: Display -> GC -> Pixel          	      -> IO ()
primitive xSetFunction "XS_XSetFunction" :: Display -> GC -> GXFunction     	      -> IO ()
primitive xSetGraphicsExposures "XS_XSetGraphicsExposures" :: Display -> GC -> Bool                  -> IO ()
primitive xSetClipMask "XS_XSetClipMask" :: Display -> GC -> Pixmap                -> IO ()
primitive xSetClipOrigin "XS_XSetClipOrigin" :: Display -> GC -> Position -> Position  -> IO ()
primitive xSetDashes "XS_XSetDashes" :: Display -> GC -> Int -> String -> Int  -> IO ()
primitive xSetFillRule "XS_XSetFillRule" :: Display -> GC -> FillRule              -> IO ()
primitive xSetFillStyle "XS_XSetFillStyle" :: Display -> GC -> FillStyle             -> IO ()
primitive xSetFont "XS_XSetFont" :: Display -> GC -> Font                  -> IO ()
primitive xSetLineAttributes "XS_XSetLineAttributes" :: Display -> GC -> Int -> LineStyle -> CapStyle -> JoinStyle -> IO ()
primitive xSetPlaneMask "XS_XSetPlaneMask" :: Display -> GC -> Pixel                 -> IO ()
primitive xSetState "XS_XSetState" :: Display -> GC -> Pixel -> Pixel -> GXFunction -> Pixel -> IO ()
primitive xSetStipple "XS_XSetStipple" :: Display -> GC -> Pixmap                -> IO ()
primitive xSetSubwindowMode "XS_XSetSubwindowMode" :: Display -> GC -> SubWindowMode         -> IO ()
primitive xSetTSOrigin "XS_XSetTSOrigin" :: Display -> GC -> Position -> Position  -> IO ()
primitive xSetTile "XS_XSetTile" :: Display -> GC -> Pixmap                -> IO ()
primitive xCreateGC "XS_XCreateGC" :: Display -> Drawable -> IO GC
primitive xFreeGC "XS_XFreeGC" :: Display -> GC -> IO ()
primitive xFlushGC "XS_XFlushGC" :: Display -> GC -> IO ()
primitive xCreatePixmap "XS_XCreatePixmap" :: Display -> Drawable -> Dimension -> Dimension -> Int -> IO Pixmap
primitive xFreePixmap "XS_XFreePixmap" :: Display -> Pixmap -> IO ()
primitive xBitmapBitOrder "XS_XBitmapBitOrder" :: Display -> ByteOrder
primitive xBitmapUnit "XS_XBitmapUnit" :: Display -> Int
primitive xBitmapPad "XS_XBitmapPad" :: Display -> Int
primitive xLookupColor "XS_XLookupColor" :: Display -> Colormap -> String -> IO (XColor, XColor)     
primitive xAllocNamedColor "XS_XAllocNamedColor" :: Display -> Colormap -> String -> IO (XColor, XColor) 
primitive xAllocColor "XS_XAllocColor" :: Display -> Colormap -> XColor -> IO XColor                
primitive xParseColor "XS_XParseColor" :: Display -> Colormap -> String -> IO XColor                
primitive xFreeColors "XS_XFreeColors" :: Display -> Colormap -> ListPixel -> Pixel -> IO ()
primitive xStoreColor "XS_XStoreColor" :: Display -> Colormap -> XColor -> IO ()
primitive xStoreColors "XS_XStoreColors" :: Display -> Colormap -> ListXColor -> IO ()
primitive xStoreNamedColor "XS_XStoreNamedColor" :: Display -> Colormap -> String -> Pixel -> PrimaryMask -> IO ()
primitive xQueryColor "XS_XQueryColor" :: Display -> Colormap -> XColor -> IO XColor
primitive xQueryColors "XS_XQueryColors" :: Display -> Colormap -> ListXColor -> IO ListXColor
primitive xInstallColormap "XS_XInstallColormap" :: Display -> Colormap -> IO ()
primitive xUninstallColormap "XS_XUninstallColormap" :: Display -> Colormap -> IO ()
primitive xCopyColormapAndFree "XS_XCopyColormapAndFree" :: Display -> Colormap -> IO Colormap
primitive xCreateColormap "XS_XCreateColormap" :: Display -> Window   -> Visual -> ColormapAlloc -> IO Colormap
primitive xFreeColormap "XS_XFreeColormap" :: Display -> Colormap -> IO ()
newtype Glyph = Glyph Int deriving ()
primitive xQueryFont "XS_XQueryFont" :: Display -> Font -> IO XFontStruct
primitive xLoadQueryFont "XS_XLoadQueryFont" :: Display -> String -> IO XFontStruct
primitive xFreeFont "XS_XFreeFont" :: Display -> XFontStruct -> IO ()
primitive xDisplayKeycodes "XS_XDisplayKeycodes" :: Display -> (Int,Int)
primitive xLookupKeysym "XS_XLookupKeysym" :: XKeyEventPtr -> Int -> IO KeySym
primitive xKeycodeToKeysym "XS_XKeycodeToKeysym" :: Display -> KeyCode -> Int -> IO KeySym
primitive xKeysymToKeycode "XS_XKeysymToKeycode" :: Display -> KeySym  -> IO KeyCode
primitive xKeysymToString "XS_XKeysymToString" :: KeySym -> String
primitive xStringToKeysym "XS_XStringToKeysym" :: String -> KeySym
primitive xGetIconName "XS_XGetIconName" :: Display -> Window -> IO String 
primitive xSetIconName "XS_XSetIconName" :: Display -> Window -> String -> IO ()
primitive xDefineCursor "XS_XDefineCursor" :: Display -> Window -> Cursor -> IO ()
primitive xUndefineCursor "XS_XUndefineCursor" :: Display -> Window -> IO ()
primitive xCreatePixmapCursor "XS_XCreatePixmapCursor" :: Display -> Pixmap -> Pixmap -> XColor -> XColor -> Dimension -> Dimension -> IO Cursor
primitive xCreateGlyphCursor "XS_XCreateGlyphCursor" :: Display -> Font   -> Font -> Glyph -> Glyph -> XColor -> XColor -> IO Cursor
primitive xCreateFontCursor "XS_XCreateFontCursor" :: Display -> Shape  -> IO Cursor
primitive xFreeCursor "XS_XFreeCursor" :: Display -> Font   -> IO ()
primitive xRecolorCursor "XS_XRecolorCursor" :: Display -> Cursor -> XColor -> XColor -> IO ()
type XSetWindowAttributesPtr = Ptr in
  toXSetWindowAttributesPtr   :: Ptr -> XSetWindowAttributesPtr,
  fromXSetWindowAttributesPtr :: XSetWindowAttributesPtr -> Ptr

toXSetWindowAttributesPtr   x = x
fromXSetWindowAttributesPtr x = x

instance Pointer XSetWindowAttributesPtr where
  toPtr     = fromXSetWindowAttributesPtr
  fromPtr   = toXSetWindowAttributesPtr
  alloc     = malloc sizeofXSetWindowAttributes
  plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofXSetWindowAttributes))
  minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofXSetWindowAttributes
sizeofXSetWindowAttributes :: Int
sizeofXSetWindowAttributes = consts_Int 2
primitive set_background_pixmap "XS_set_background_pixmap" :: XSetWindowAttributesPtr -> Pixmap -> IO ()
primitive set_background_pixel "XS_set_background_pixel" :: XSetWindowAttributesPtr -> Pixel -> IO ()
primitive set_border_pixmap "XS_set_border_pixmap" :: XSetWindowAttributesPtr -> Pixmap -> IO ()
primitive set_border_pixmel "XS_set_border_pixmel" :: XSetWindowAttributesPtr -> Pixel -> IO ()
primitive set_bit_gravity "XS_set_bit_gravity" :: XSetWindowAttributesPtr -> BitGravity -> IO ()
primitive set_win_gravity "XS_set_win_gravity" :: XSetWindowAttributesPtr -> WindowGravity -> IO ()
primitive set_backing_store "XS_set_backing_store" :: XSetWindowAttributesPtr -> BackingStore -> IO ()
primitive set_backing_planes "XS_set_backing_planes" :: XSetWindowAttributesPtr -> Pixel -> IO ()
primitive set_backing_pixel "XS_set_backing_pixel" :: XSetWindowAttributesPtr -> Pixel -> IO ()
primitive set_save_under "XS_set_save_under" :: XSetWindowAttributesPtr -> Bool -> IO ()
primitive set_event_mask "XS_set_event_mask" :: XSetWindowAttributesPtr -> EventMask -> IO ()
primitive set_do_not_propagate_mask "XS_set_do_not_propagate_mask" :: XSetWindowAttributesPtr -> EventMask -> IO ()
primitive set_overide_redirect "XS_set_overide_redirect" :: XSetWindowAttributesPtr -> Bool -> IO ()
primitive set_colormap "XS_set_colormap" :: XSetWindowAttributesPtr -> Colormap -> IO ()
primitive set_cursor "XS_set_cursor" :: XSetWindowAttributesPtr -> Cursor -> IO ()
primitive xStoreName "XS_XStoreName" :: Display -> Window -> String -> IO ()
primitive xCreateSimpleWindow "XS_XCreateSimpleWindow" :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> Int -> Pixel -> Pixel -> IO Window
primitive xCreateWindow "XS_XCreateWindow" :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> Int -> Int -> WindowClass -> Visual -> AttributeMask -> XSetWindowAttributesPtr -> IO Window
primitive xTranslateCoordinates "XS_XTranslateCoordinates" :: Display -> Window -> Window -> Position -> Position -> IO (Bool,Position,Position,Window)
primitive xMoveResizeWindow "XS_XMoveResizeWindow" :: Display -> Window -> Position  -> Position  -> Dimension -> Dimension -> IO ()
primitive xResizeWindow "XS_XResizeWindow" :: Display -> Window -> Dimension -> Dimension -> IO ()
primitive xMoveWindow "XS_XMoveWindow" :: Display -> Window -> Position  -> Position  -> IO ()
primitive xReparentWindow "XS_XReparentWindow" :: Display -> Window -> Window -> Position -> Position  -> IO ()
primitive xMapSubwindows "XS_XMapSubwindows" :: Display -> Window -> IO ()
primitive xUnmapSubwindows "XS_XUnmapSubwindows" :: Display -> Window -> IO ()
primitive xMapWindow "XS_XMapWindow" :: Display -> Window -> IO ()
primitive xLowerWindow "XS_XLowerWindow" :: Display -> Window -> IO ()
primitive xRaiseWindow "XS_XRaiseWindow" :: Display -> Window -> IO ()
primitive xCirculateSubwindowsDown "XS_XCirculateSubwindowsDown" :: Display -> Window -> IO ()
primitive xCirculateSubwindowsUp "XS_XCirculateSubwindowsUp" :: Display -> Window -> IO ()
primitive xCirculateSubwindows "XS_XCirculateSubwindows" :: Display -> Window -> CirculationDirection -> IO ()
primitive xIconifyWindow "XS_XIconifyWindow" :: Display -> Window -> ScreenNumber -> IO ()
primitive xWithdrawWindow "XS_XWithdrawWindow" :: Display -> Window -> ScreenNumber -> IO ()
primitive xDestroyWindow "XS_XDestroyWindow" :: Display -> Window -> IO ()
primitive xDestroySubwindows "XS_XDestroySubwindows" :: Display -> Window -> IO ()
primitive xSetWindowBorder "XS_XSetWindowBorder" :: Display -> Window -> Pixel     -> IO ()
primitive xSetWindowBorderPixmap "XS_XSetWindowBorderPixmap" :: Display -> Window -> Pixmap    -> IO ()
primitive xSetWindowBorderWidth "XS_XSetWindowBorderWidth" :: Display -> Window -> Dimension -> IO ()
primitive xSetWindowBackground "XS_XSetWindowBackground" :: Display -> Window -> Pixel     -> IO ()
primitive xSetWindowBackgroundPixmap "XS_XSetWindowBackgroundPixmap" :: Display -> Window -> Pixmap    -> IO ()
primitive xSetWindowColormap "XS_XSetWindowColormap" :: Display -> Window -> Colormap  -> IO ()
primitive xAddToSaveSet "XS_XAddToSaveSet" :: Display -> Window -> IO ()
primitive xRemoveFromSaveSet "XS_XRemoveFromSaveSet" :: Display -> Window -> IO ()
primitive xChangeSaveSet "XS_XChangeSaveSet" :: Display -> Window -> ChangeSaveSetMode -> IO ()
primitive xClearWindow "XS_XClearWindow" :: Display -> Window -> IO ()
primitive xClearArea "XS_XClearArea" :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> Bool -> IO ()
primitive xQueryTree "XS_XQueryTree" :: Display -> Window -> IO (Window, Window, ListWindow)
primitive xRestackWindows "XS_XRestackWindows" :: Display -> ListWindow -> IO ()
primitive xListInstalledColormaps "XS_XListInstalledColormaps" :: Display -> Window -> IO ListColormap
primitive xGetTransientForHint "XS_XGetTransientForHint" :: Display -> Window -> IO Window 
primitive xSetTransientForHint "XS_XSetTransientForHint" :: Display -> Window -> Window -> IO ()
primitive xDrawPoint "XS_XDrawPoint" :: Display -> Drawable -> GC -> Position -> Position -> IO ()
primitive xDrawPoints "XS_XDrawPoints" :: Display -> Drawable -> GC -> ListXPoint -> CoordinateMode -> IO ()
primitive xDrawLine "XS_XDrawLine" :: Display -> Drawable -> GC -> Position -> Position -> Position -> Position -> IO ()
primitive xDrawLines "XS_XDrawLines" :: Display -> Drawable -> GC -> ListXPoint -> CoordinateMode -> IO ()
primitive xDrawSegments "XS_XDrawSegments" :: Display -> Drawable -> GC -> ListXSegment -> IO ()
primitive xDrawRectangle "XS_XDrawRectangle" :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> IO ()
primitive xDrawRectangles "XS_XDrawRectangles" :: Display -> Drawable -> GC -> ListXRectangle -> IO ()
primitive xDrawArc "XS_XDrawArc" :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Int -> Int -> IO ()
primitive xDrawArcs "XS_XDrawArcs" :: Display -> Drawable -> GC -> ListXArc -> IO ()
primitive xFillRectangle "XS_XFillRectangle" :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> IO ()
primitive xFillRectangles "XS_XFillRectangles" :: Display -> Drawable -> GC -> ListXRectangle -> IO ()
primitive xFillPolygon "XS_XFillPolygon" :: Display -> Drawable -> GC -> ListXPoint -> PolygonShape -> CoordinateMode -> IO ()
primitive xFillArc "XS_XFillArc" :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Int -> Int -> IO ()
primitive xFillArcs "XS_XFillArcs" :: Display -> Drawable -> GC -> ListXArc -> IO ()
primitive xCopyArea "XS_XCopyArea" :: Display -> Drawable -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Position -> Position -> IO ()
primitive xCopyPlane "XS_XCopyPlane" :: Display -> Drawable -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Position -> Position -> Pixel -> IO ()
primitive xDrawString "XS_XDrawString" :: Display -> Drawable -> GC -> Position -> Position -> String -> IO ()
primitive xDrawImageString "XS_XDrawImageString" :: Display -> Drawable -> GC -> Position -> Position -> String -> IO ()
primitive consts_AttributeMask "XSconsts_AttributeMask" :: Int -> AttributeMask
primitive consts_JoinStyle "XSconsts_JoinStyle" :: Int -> JoinStyle
primitive consts_CapStyle "XSconsts_CapStyle" :: Int -> CapStyle
primitive consts_BitGravity "XSconsts_BitGravity" :: Int -> BitGravity
primitive consts_Place "XSconsts_Place" :: Int -> Place
primitive consts_ScreenSaverMode "XSconsts_ScreenSaverMode" :: Int -> ScreenSaverMode
primitive consts_Visibility "XSconsts_Visibility" :: Int -> Visibility
primitive consts_VisualClass "XSconsts_VisualClass" :: Int -> VisualClass
primitive consts_PropertyNotification "XSconsts_PropertyNotification" :: Int -> PropertyNotification
primitive consts_WindowClass "XSconsts_WindowClass" :: Int -> WindowClass
primitive consts_EventMask "XSconsts_EventMask" :: Int -> EventMask
primitive consts_RectangleOrdering "XSconsts_RectangleOrdering" :: Int -> RectangleOrdering
primitive consts_PrimaryMask "XSconsts_PrimaryMask" :: Int -> PrimaryMask
primitive consts_AllowExposuresMode "XSconsts_AllowExposuresMode" :: Int -> AllowExposuresMode
primitive consts_AllowEvents "XSconsts_AllowEvents" :: Int -> AllowEvents
primitive consts_QueryBestSizeClass "XSconsts_QueryBestSizeClass" :: Int -> QueryBestSizeClass
primitive consts_KeyMask "XSconsts_KeyMask" :: Int -> KeyMask
primitive consts_LineStyle "XSconsts_LineStyle" :: Int -> LineStyle
primitive consts_LedMode "XSconsts_LedMode" :: Int -> LedMode
primitive consts_BackingStore "XSconsts_BackingStore" :: Int -> BackingStore
primitive consts_Button "XSconsts_Button" :: Int -> Button
primitive consts_NotifyMode "XSconsts_NotifyMode" :: Int -> NotifyMode
primitive consts_WindowGravity "XSconsts_WindowGravity" :: Int -> WindowGravity
primitive consts_FillRule "XSconsts_FillRule" :: Int -> FillRule
primitive consts_AutoRepeatMode "XSconsts_AutoRepeatMode" :: Int -> AutoRepeatMode
primitive consts_GrabStatus "XSconsts_GrabStatus" :: Int -> GrabStatus
primitive consts_GrabMode "XSconsts_GrabMode" :: Int -> GrabMode
primitive consts_EventType "XSconsts_EventType" :: Int -> EventType
primitive consts_ColormapNotification "XSconsts_ColormapNotification" :: Int -> ColormapNotification
primitive consts_Status "XSconsts_Status" :: Int -> Status
primitive consts_GXFunction "XSconsts_GXFunction" :: Int -> GXFunction
primitive consts_FocusMode "XSconsts_FocusMode" :: Int -> FocusMode
primitive consts_Gravity "XSconsts_Gravity" :: Int -> Gravity
primitive consts_ChangeSaveSetMode "XSconsts_ChangeSaveSetMode" :: Int -> ChangeSaveSetMode
primitive consts_SubWindowMode "XSconsts_SubWindowMode" :: Int -> SubWindowMode
primitive consts_ButtonMask "XSconsts_ButtonMask" :: Int -> ButtonMask
primitive consts_ColormapAlloc "XSconsts_ColormapAlloc" :: Int -> ColormapAlloc
primitive consts_Int "XSconsts_Int" :: Int -> Int
primitive consts_NotifyDetail "XSconsts_NotifyDetail" :: Int -> NotifyDetail
primitive consts_ImageFormat "XSconsts_ImageFormat" :: Int -> ImageFormat
primitive consts_CoordinateMode "XSconsts_CoordinateMode" :: Int -> CoordinateMode
primitive consts_WindowStackingMethod "XSconsts_WindowStackingMethod" :: Int -> WindowStackingMethod
primitive consts_QueuedMode "XSconsts_QueuedMode" :: Int -> QueuedMode
primitive consts_PreferBlankingMode "XSconsts_PreferBlankingMode" :: Int -> PreferBlankingMode
primitive consts_Modifier "XSconsts_Modifier" :: Int -> Modifier
primitive consts_ByteOrder "XSconsts_ByteOrder" :: Int -> ByteOrder
primitive consts_Protocol "XSconsts_Protocol" :: Int -> Protocol
primitive consts_Access "XSconsts_Access" :: Int -> Access
primitive consts_CloseDownMode "XSconsts_CloseDownMode" :: Int -> CloseDownMode
primitive consts_MappingRequest "XSconsts_MappingRequest" :: Int -> MappingRequest
primitive consts_FillStyle "XSconsts_FillStyle" :: Int -> FillStyle
primitive consts_PolygonShape "XSconsts_PolygonShape" :: Int -> PolygonShape
primitive consts_ArcMode "XSconsts_ArcMode" :: Int -> ArcMode
primitive consts_CirculationDirection "XSconsts_CirculationDirection" :: Int -> CirculationDirection

needPrims_hugs
